using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Bwg.Asf
{
    public class AsfReader
    {
        #region private member variables
        private string m_filename;
        private FileStream m_stream;
        private bool m_close_stream;
        #endregion

        #region constructors
        public AsfReader()
        {
            m_filename = string.Empty;
            m_stream = null;
        }

        public AsfReader(string filename, FileStream strm)
        {
            m_filename = filename;
            m_stream = strm;
            m_close_stream = false;
        }

        public AsfReader(string filename)
        {
            m_filename = filename;
            m_stream = new FileStream(filename, FileMode.Open, FileAccess.Read);
            m_close_stream = true;
        }
        #endregion

        #region public members
        public void ReadFile()
        {
            ProbeFile();
        }
        #endregion

        #region private members

        private int ReadS32(string filename, FileStream strm)
        {
            int value = 0;
            byte[] length = new byte[4];

            //
            // Now read the signed 32 bit integer from the file
            //
            try
            {
                int n = strm.Read(length, 0, length.GetLength(0));
                if (n < length.GetLength(0))
                {
                    string str = "error reading signed 32 bit integer from file '" + filename + "' - ";
                    str += "premature EOF";
                    throw new Exception(str);
                }
            }
            catch (Exception ex)
            {
                string str = "error reading signed 32 bit integer from file '" + filename + "' - ";
                str += ex.Message;
                throw new Exception(str);
            }

            value |= (int)(length[0] << 0);
            value |= (int)(length[1] << 8);
            value |= (int)(length[2] << 16);
            value |= (int)(length[3] << 24);

            return value;
        }

        private long ReadS64(string filename, FileStream strm)
        {
            long value = 0;
            byte[] length = new byte[8];

            //
            // Now read the signed 64 bit integer from the file
            //
            try
            {
                int n = strm.Read(length, 0, length.GetLength(0));
                if (n < length.GetLength(0))
                {
                    string str = "error reading signed 64 bit integer from file '" + filename + "' - ";
                    str += "premature EOF";
                    throw new Exception(str);
                }
            }
            catch (Exception ex)
            {
                string str = "error reading signed 64 bit integer from file '" + filename + "' - ";
                str += ex.Message;
                throw new Exception(str);
            }

            value |= (long)(length[0] << 0);
            value |= (long)(length[1] << 8);
            value |= (long)(length[2] << 16);
            value |= (long)(length[3] << 24);
            value |= (long)(length[4] << 32);
            value |= (long)(length[5] << 40);
            value |= (long)(length[6] << 48);
            value |= (long)(length[7] << 56);

            return value;
        }

        AsfObject ReadObjectHeader(string filename, FileStream strm)
        {
            byte[] guid = new byte[16];
            byte[] length = new byte[8];

            //
            // Read the GUID from the file
            //
            try
            {
                int n = strm.Read(guid, 0, guid.GetLength(0));
                if (n < guid.GetLength(0))
                {
                    string str = "error reading GUID for next object from file '" + filename + "' - ";
                    str += "premature EOF";
                    throw new Exception(str);
                }
            }
            catch (Exception ex)
            {
                string str = "error reading GUID for next object from file '" + filename + "' - ";
                str += ex.Message;
                throw new Exception(str);
            }


            long objlen = ReadS64(filename, strm);
            return new AsfObject(new Guid(guid), objlen);
        }

        private void SkipObject(string filename, FileStream strm, AsfObject obj)
        {
            long here = strm.Position;

            if (strm.Seek(obj.MyLength - 24, SeekOrigin.Current) != here + obj.MyLength - 24)
            {
                string msg;

                msg = "seek past current object filed, premature EOF most likely";
                throw new Exception(msg);
            }
        }

        private void ParseHeader(string filename, FileStream strm)
        {
            uint objcnt = ReadS32(filename, strm);
            byte bval;

            bval = ReadU8(filename, strm);
            bval = ReadU8(filename, strm);
            if (bval != 2)
            {
                string msg = "invalid object header for ASF file";
                throw new Exception(msg);
            }

            while (objcnt--)
            {
                AsfObject obj = ReadObjectHeader(filename, strm);
                SkilObject(obj);
            }
        }

        private void ProbeFile(string filename, FileStream strm)
        {
            AsfObject obj;

            obj = ReadObjectHeader(filename, strm);
            if (obj.MyGuid != ASF_Header_Object)
            {
                string msg;

                msg = "invalid file format, expected ASF_Header_Object, guid=";
                msg += ASF_Header_Object.ToString() + ", got guid = ";
                msg += obj.MyGuid.ToString();
                throw new Exception(msg);
            }
            long here = strm.Position;
            ParseHeader();
            strm.Seek(here, SeekOrigin.Begin);
            SkipObject(filename, strm, obj);

            obj = ReadObjectHeader(filename, strm);
            if (obj.MyGuid != ASF_Data_Object)
            {
                string msg;

                msg = "invalid file format, expected ASF_Data_Object, guid=";
                msg += ASF_Data_Object.ToString() + ", got guid = ";
                msg += obj.MyGuid.ToString();
                throw new Exception(msg);
            }
        }
        #endregion
    }
}
