//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel ;
using System.Runtime.InteropServices;
using Bwg.Scsi ;
using Bwg.Audio ;

namespace Bwg.Audio.Ripper
{
    class CDAudioReader : AudioReader
    {
        [DllImport("BwgAudioScanner.dll")]
        internal static extern bool BwgCheckZero(IntPtr data, int count);

        public ProgressChangedEventHandler Progress;

        private Device m_device;
        private int m_start;
        private int m_length;
        private int m_pos;
        private bool m_speed_set;
        private int m_percent;

        public CDAudioReader(Device d, int lba, int length)
        {
            m_device = d;
            m_start = lba;
            m_length = length;
            m_pos = lba;

            m_percent = 0;

            m_number_of_channels = 2;
            m_bits_per_channel = 16;
            m_sample_rate = 44100;
            m_speed_set = false;
        }

        public override int PreferredReadSize
        {
            get
            {
                return (m_device.MaximumTransferLength / 2352) * 2352;
            }
        }

        public override bool ScanInputFile(out short lmin, out short lmax, out short rmin, out short rmax)
        {
            lmin = 0;
            lmax = 0;
            rmin = 0;
            rmax = 0;
            return true;
        }

        public override long ByteCount
        {
            get
            {
                return m_length * 2352;
            }
        }

        public override string FileTypeString
        {
            get
            {
                return "CD Audio";
            }
        }

        public override string Name
        {
            get
            {
                return "CD Audio Ripper";
            }
        }

        public override bool Rewind()
        {
            m_pos = m_start;
            return true;
        }

        public override int Read(IntPtr data, int count)
        {
            Device.CommandStatus st;
            int maxsect = m_device.MaximumTransferLength / 2352;
            int sect = count / 2352;

            if (Progress != null)
            {
                int pcnt = (int)((m_pos - m_start) * 100 / m_length);
                if (pcnt < 0)
                    pcnt = 0;
                if (pcnt > 100)
                    pcnt = 100;
                if (pcnt != m_percent)
                {
                    ProgressChangedEventArgs args = new ProgressChangedEventArgs(pcnt, null);
                    Progress(this, args);
                    m_percent = pcnt;
                }
            }

            if (m_pos - m_start == m_length)
                return 0;

            if (m_pos - m_start + sect > m_length)
                sect = m_length - m_pos + m_start;

            if (sect > maxsect)
                sect = maxsect;

            if (!m_speed_set)
            {
                m_device.SetCdSpeed(Device.RotationalControl.PureCav, Device.OptimumSpeed, Device.OptimumSpeed);
                m_speed_set = true;
            }

            st = m_device.ReadCD(1, true, (uint)m_pos, (uint)sect, data, count);
            if (st != Device.CommandStatus.Success)
                return 0;

            m_pos += sect;
            return sect * 2352;
        }
    }
}
