//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using Bwg.Audio.Acm ;

namespace Bwg.Audio
{
    internal abstract class AudioHeaderReader
    {
        #region protected member variables

        /// <summary>
        /// This is a string describing the type of the file.  Generally it should
        /// contains more than just the type, it should contain some information about
        /// the format as well.  This string is used when the file type needs to be
        /// displayed to the user.
        /// </summary>
        protected string m_file_type_string;

        /// <summary>
        /// This is the position of the raw audio data within the file. The
        /// data is the raw data that will be handed to the ACM functions to 
        /// convert to our desired format
        /// </summary>
        protected long m_audio_data_position;

        /// <summary>
        /// This is the WAVEFORMATEX (or derived) structure that defines the format
        /// of the data found in the file.  The WAVEFORMATEX structure is represented
        /// as a series of bytes in a byte array.
        /// </summary>
        protected byte[] m_data_format;

        /// <summary>
        /// This is the size of the buffer that is ideal during acmStreamConvert() as an
        /// input buffer for this data type
        /// </summary>
        protected uint m_buffer_size;

        /// <summary>
        /// This is the ACM manager and is used to query support for a given format and to
        /// query for the appropriate formats to use
        /// </summary>
        protected AcmManager m_acm_manager;

#endregion

        #region constructors
        public AudioHeaderReader(AcmManager mgr)
        {
            m_acm_manager = mgr;
            m_file_type_string = "Unknown File Format";
            m_data_format = null;
            m_buffer_size = 0;
        }
        #endregion

        #region public properties
        /// <summary>
        /// This property returns the position of the raw audio data within the file.
        /// </summary>
        public long AudioDataPosition
        {
            get
            {
                return m_audio_data_position;
            }
        }

        /// <summary>
        /// This property retuns the format of the data in the file in the form of a
        /// WAVEFORMATEX (or derived) structure.  The data is returned as a seried of
        /// bytes in a byte array.
        /// </summary>
        public byte[] DataFormat
        {
            get
            {
                return m_data_format;
            }
        }

        /// <summary>
        /// This property is the size of the input buffer that is ideal (required) for the
        /// given type of data.
        /// </summary>
        public uint BufferSize
        {
            get
            {
                return m_buffer_size;
            }
        }

        /// <summary>
        /// This is a string describing the type of the file.  Generally it should
        /// contains more than just the type, it should contain some information about
        /// the format as well.  This string is used when the file type needs to be
        /// displayed to the user.
        /// </summary>
        public string FileTypeString
        {
            get
            {
                return m_file_type_string;
            }
        }


        #endregion

        #region public methods
        /// <summary>
        /// This method parse the header of the file and sets the properties
        /// associated with the header of the file.
        /// </summary>
        /// <param name="filename"></param>
        public abstract void ReadHeader(string filename);
        #endregion
    }
}
