using System;
using System.Collections.Generic;
using System.Text;

namespace Bwg.Burner
{
    /// <summary>
    /// This class represnts a set of binary values
    /// </summary>
    [Serializable()]
    public class CDTextDataBinary : CDTextData
    {
        #region private member variables
        private byte[] m_data;
        #endregion

        #region constructor
        /// <summary>
        /// Create a new binary data object from a series of bytes
        /// </summary>
        /// <param name="data"></param>
        public CDTextDataBinary(byte[] data)
        {
            m_data = new byte[data.GetLength(0)];
            Array.Copy(data, m_data, data.GetLength(0));
        }

        /// <summary>
        /// Create a new empty binary data object
        /// </summary>
        public CDTextDataBinary()
        {
            m_data = new byte[0];
        }
        #endregion

        #region public member functions
        /// <summary>
        /// Convert a binary array to a string
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            StringBuilder bld = new StringBuilder();

            bool first = true;
            foreach (byte b in m_data)
            {
                if (!first)
                    bld.Append(' ');
                bld.Append(b.ToString("X2"));
                first = false;
            }

            return bld.ToString();
        }

        /// <summary>
        /// Add data bytes to this binary cd text data value
        /// </summary>
        /// <param name="pos">the position in the data stream</param>
        /// <param name="data">the amount of data to add</param>
        public override void AddData(byte pos, byte[] data)
        {
            int len = m_data.GetLength(0) ;
            Array.Resize(ref m_data, len + data.GetLength(0));
            Array.Copy(data, 0, m_data, len, data.GetLength(0));
        }
        #endregion

        #region public properties

        /// <summary>
        /// Return the data bytes associated with this cd text field data value
        /// </summary>
        public override byte[] Data
        {
            get
            {
                return m_data;
            }
        }

        /// <summary>
        /// Return a data byte from this binary data value
        /// </summary>
        /// <param name="index">the index of the byte to return</param>
        /// <returns>the byte of data</returns>
        public byte this[int index]
        {
            get
            {
                return m_data[index];
            }
        }

        /// <summary>
        /// Retuern the length of the data in bytes
        /// </summary>
        public override int DataLength
        {
            get
            {
                return m_data.GetLength(0);
            }
        }
        #endregion
    }
}
