using System;
using System.Collections.Generic;
using System.Text;

namespace Bwg.Burner
{
    /// <summary>
    /// This is the base class for all DVD operations.  It basically handles all of the write
    /// protected queries and returns the fact that for DVD write protection is not supported.
    /// </summary>
    public abstract class DVDBaseDiskOperations : DiskOperations
    {
        /// <summary>
        /// Create the base class DVD operations
        /// </summary>
        /// <param name="d">the drive for the operation</param>
        public DVDBaseDiskOperations(Drive d) : base(d)
        {
        }

        /// <summary>
        /// Returns false, as by default DVD media does not support format (yet)
        /// </summary>
        public override bool IsFormatSupported
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Returns true if write protect is supports (and it is for this media)
        /// </summary>
        public override bool IsWriteProtectSupported
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// This property returns true if the loaded disk is write protected
        /// </summary>
        public override bool IsWriteProtected
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// This property returns true if the loaded disk has a write protect password
        /// </summary>
        public override bool HasWriteProtectPassword
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// This property provides the write protect password for a given media
        /// </summary>
        public override string WriteProtectPassword
        {
            set
            {
                throw new Exception("Cannot supply write protect password for this type of media");
            }
        }
    }
}
