//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using Bwg.Scsi ;

namespace Bwg.Burner
{
    /// <summary>
    /// This class is an abstract class that represents a source of data
    /// for the disk to be burned
    /// </summary>
    public abstract class DataSource
    {
        /// <summary>
        /// Return the size of the data source in bytes
        /// </summary>
        public abstract long SectorCount
        {
            get;
        }

        /// <summary>
        /// Returns the name of the data source
        /// </summary>
        public abstract string Name
        {
            get;
        }

        /// <summary>
        /// Send data into the pool, it must be an integral number of sectors
        /// </summary>
        /// <param name="pool">the pool to send pages into</param>
        /// <param name="start_sector">the staring sector for this data source</param>
        /// <param name="sector_size">the size of the sectors to send in bytes</param>
        /// <param name="lba">the address of the blocks to send to the device</param>
        /// <returns>the number of pages sent</returns>
        public abstract long SendData(WriteBufferPool pool, long start_sector, int sector_size, long lba);
    }
}
