//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.IO ;
using System.Diagnostics;
using Bwg.Scsi;

namespace Bwg.Burner
{
    /// <summary>
    /// A data source where the data comes from a file.
    /// </summary>
    public class FileDataSource : DataSource
    {
        #region private data members
        private FileInfo m_info;
        #endregion

        #region constructors
        /// <summary>
        /// Create a file data source object given the name of the file to
        /// stream to the disk burner
        /// </summary>
        /// <param name="filename"></param>
        public FileDataSource(string filename)
        {
            m_info = new FileInfo(filename);
        }
        #endregion

        #region public properties

        /// <summary>
        /// This property returns the name of the data source (actually just the
        /// name of the file)
        /// </summary>
        public override string Name
        {
            get
            {
                return m_info.Name;
            }
        }

        /// <summary>
        /// Return the size of the file data in bytes
        /// </summary>
        /// <returns></returns>
        public override long SectorCount
        {
            get
            {
                // As this is an image file, the sector size is 2048
                long cnt = m_info.Length / 2048;
                if ((m_info.Length % 2048) != 0)
                    cnt++;

                return cnt;
            }
        }
        #endregion

        #region public methods
        /// <summary>
        /// Send data down to the burner
        /// </summary>
        /// <param name="pool"></param>
        /// <param name="start_sector">the starting sector for this data source</param>
        /// <param name="sector_size">the size of the sectors to send in bytes</param>
        /// <param name="lba">the starting logical block address for the file data</param>
        /// <returns></returns>
        public override long SendData(Bwg.Scsi.WriteBufferPool pool, long start_sector, int sector_size, long lba)
        {
            FileReader reader = new FileReader(pool);
            if (!reader.Open(m_info.FullName))
                throw new Exception("cannot open file '" + m_info.FullName + "'");

            if (!reader.ReadData(sector_size, lba))
                throw new Exception("read error");

            long sectors = m_info.Length / sector_size;
            if ((m_info.Length % sector_size) != 0)
                sectors++;

            reader.Close();

            Debug.Assert(sectors < Int32.MaxValue);
            return sectors;
        }
        #endregion
    }
}
