//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Bwg.Scsi ;

namespace Bwg.Burner
{
    /// <summary>
    /// This class represents a single speed setting for either read or write
    /// </summary>
    public class SpeedInfo
    {
        #region private member variables

        /// <summary>
        /// The speed descriptor for this wpeed
        /// </summary>
        SpeedDescriptor m_desc;

        /// <summary>
        /// This member indicates whether this speed descriptor is for a CD
        /// or a DVD.  
        /// </summary>
        private bool m_cd;

        /// <summary>
        /// The rotational control for this speed
        /// </summary>
        private Device.RotationalControl m_rot;

        /// <summary>
        /// The speed for this speed setting
        /// </summary>
        private ushort m_speed;
        #endregion

        #region constructor
        /// <summary>
        /// Create the speed info object from a speed descriptor
        /// </summary>
        /// <param name="desc">the speed descriptor for this entry</param>
        /// <param name="iscd">true if this descriptor applies to a CD, false if it applies to DVD</param>
        public SpeedInfo(SpeedDescriptor desc, bool iscd)
        {
            m_desc = desc;
            m_cd = iscd;
        }

        /// <summary>
        /// Create the speed info object from a mode page speed setting
        /// </summary>
        /// <param name="rot"></param>
        /// <param name="speed"></param>
        /// <param name="iscd"></param>
        public SpeedInfo(Device.RotationalControl rot, ushort speed, bool iscd)
        {
            m_desc = null;
            m_rot = rot;
            m_speed = speed;
            m_cd = iscd;
        }

        #endregion

        #region public properties

        /// <summary>
        /// Return the speed value for this speed
        /// </summary>
        public ushort Value
        {
            get
            {
                Debug.Assert(IsModePage);
                return m_speed;
            }
        }

        /// <summary>
        /// Returns the rotational control for this speed
        /// </summary>
        public Device.RotationalControl Rot
        {
            get
            {
                Debug.Assert(IsModePage);
                return m_rot;
            }
        }

        /// <summary>
        /// Returns true if this data came from a mode page
        /// </summary>
        public bool IsModePage
        {
            get
            {
                return m_desc == null;
            }
        }

        /// <summary>
        /// If true, this speed is for a CD, otherwise it is for a DVD
        /// </summary>
        public bool IsCD
        {
            get
            {
                return m_cd;
            }
        }

        /// <summary>
        /// This property is the descriptor assocaited with this entry
        /// </summary>
        public SpeedDescriptor Desc
        {
            get
            {
                return m_desc;
            }
        }
                
        #endregion

        #region public methods
        /// <summary>
        /// This method returns a human reabable string representing this speed
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            int dr ;


            if (m_desc != null)
                dr = m_desc.WriteSpeed;
            else
                dr = m_speed ;

            double spdmul;

            if (m_cd)
                spdmul = m_desc.WriteSpeed * 1000.0 / 176400.0;
            else
                spdmul = m_desc.WriteSpeed * 1000.0 / 1385000.0;

            return spdmul.ToString("G3") + "x - " + dr.ToString() + " bytes/msec";
        }
        #endregion
    }
}
