//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using Bwg.Logging ;
using Bwg.Scsi ;

namespace Bwg.Controls
{
    public partial class DriveSelectDropDown : ComboBox
    {
        #region private constants
        private const string CreateADiskImageName = "Disk Image" ;
        #endregion

        #region private member variables
        /// <summary>
        /// The logger for logging messages
        /// </summary>
        private Logger m_logger;

        #endregion

        #region constructor
        /// <summary>
        /// Create the CD/DVD selection control
        /// </summary>
        public DriveSelectDropDown()
        {
            m_logger = null;
            InitializeComponent();
            DropDownStyle = ComboBoxStyle.DropDownList;
        }
        #endregion

        #region public properties

        /// <summary>
        /// This property returns true if the option to create a disk image is available
        /// </summary>
        public bool IsCreateImageSelected
        {
            get
            {
                return SelectedDeviceDescription == CreateADiskImageName;
            }
        }

        /// <summary>
        /// Retreive a description of the selected device
        /// </summary>
        public string SelectedDeviceDescription
        {
            get
            {
                return (string)SelectedItem;
            }
        }

        /// <summary>
        /// Retreive the device name for the selected device
        /// </summary>
        public string SelectedDeviceName
        {
            get
            {
                string desc = SelectedDeviceDescription;
                if (desc == null)
                    return null;

                return "\\\\.\\" + desc[0] + ":";
            }
        }

        /// <summary>
        /// The message logger associated with the control
        /// </summary>
        public Logger Logger
        {
            get 
            { 
                return m_logger; 
            }
            set
            {
                m_logger = value;
            }
        }

        #endregion

        #region public methods
        /// <summary>
        /// Initialize the list of devices in the control
        /// </summary>
        public void InitializeDeviceList(bool createimage)
        {
            Debug.Assert(m_logger != null);

            if (createimage)
                Items.Add(CreateADiskImageName);

            IList<string> drives = Device.GetDrives(m_logger);
            foreach (string s in drives)
            {
                if (s.Length < 5)
                    continue;

                string drv = s[4] + ": ";
                using (Device dev = new Device(m_logger))
                {
                    if (!dev.Open(s))
                        continue;

                    InquiryResult result;
                    if (dev.Inquiry(out result) != Device.CommandStatus.Success)
                        continue;

                    drv += " ";
                    drv += result.VendorIdentification;
                    drv += " ";
                    drv += result.ProductIdentification ;
                }

                Items.Add(drv);
            }

            if (Items.Count > 0)
                SelectedItem = Items[0];
        }
        #endregion
    }
}
