//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Bwg.Controls
{
    /// <summary>
    /// This class displays a block of memory as a series of hex bytes
    /// </summary>
    public partial class MemoryViewer : ListView
    {
        #region private variables
        private int m_start_addr;
        private byte[] m_data;
        #endregion

        #region properties
        /// <summary>
        /// The start address for the data to be displayed
        /// </summary>
        public int StartAddress
        {
            get
            {
                return m_start_addr;
            }
            set
            {
                m_start_addr = value;
            }
        }

        /// <summary>
        /// The data to be displayed
        /// </summary>
        public byte[] Data
        {
            get
            {
                return m_data;
            }
            set
            {
                m_data = value;
                Populate();
            }
        }

        /// <summary>
        /// The font size to use
        /// </summary>
        public float FontSize = 10.0f;

        /// <summary>
        /// The number of byte per line
        /// </summary>
        public int BytesPerLine = 16;
        #endregion

        /// <summary>
        /// Create the memory viewer
        /// </summary>
        public MemoryViewer()
        {
            InitializeComponent();

            View = View.Details;
            Font = new Font(FontFamily.GenericMonospace, FontSize);

            CreateColumns();
            Populate();
        }

        private void CreateColumns()
        {
            Columns.Add(new ColumnHeader("Addr"));
            Columns.Add(new ColumnHeader("Data"));

            Columns[0].Text = "Addr";
            Columns[1].Text = "Data";
        }

        private void Populate()
        {
            Items.Clear();

            if (Data == null)
                return;

            int addr = StartAddress;
            int index = 0;

            while (index < Data.GetLength(0))
            {
                string str = string.Empty;
                string ascii = string.Empty;
                int cnt = 0;

                ListViewItem item = new ListViewItem(addr.ToString("x8"));
                while (cnt < BytesPerLine && index < Data.GetLength(0))
                {
                    str += " ";
                    str += Data[index].ToString("x2");

                    if (Data[index] >= 0x20 && Data[index] < 0x7f)
                        ascii += (char)Data[index];
                    else
                        ascii += '.';

                    index++;
                    cnt++;
                    addr++;
                }

                while (str.Length < 3 * BytesPerLine)
                    str += ' ';

                str += "   " + ascii;

                item.SubItems.Add(str);
                Items.Add(item);
            }

            Columns[0].Width = -2;
            Columns[1].Width = Width - Columns[0].Width - 28;
        }
    }
}
