//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Bwg.Controls
{
    public partial class TaskButton : Button
    {
        private Image m_enabled_image;
        private Image m_disabled_image;
        private bool m_draw_in_color ;
        private string m_help_text;
        private Control m_buddy_control ;

        /// <summary>
        /// Create a new task button
        /// </summary>
        public TaskButton()
        {
            InitializeComponent();
        }

        /// <summary>
        /// This property is the image to display when the mouse is over the button
        /// </summary>
        public Image EnabledButtonImage
        {
            get
            {
                return m_enabled_image;
            }
            set
            {
                m_enabled_image = value;
                Invalidate();
            }
        }

        /// <summary>
        /// This property is the image to display when the mouse is not over the button
        /// </summary>
        public Image DisabledButtonImage
        {
            get
            {
                return m_disabled_image;
            }

            set
            {
                m_disabled_image = value;
                Invalidate();
            }
        }

        /// <summary>
        /// If true, draw the button is color (enabled), othersize draw in disabled mode
        /// </summary>
        public bool DrawInColor
        {
            get
            {
                return m_draw_in_color;
            }
            set
            {
                m_draw_in_color = value;
                Invalidate();
            }
        }

        /// <summary>
        /// The help text associated with this button
        /// </summary>
        public string DescriptionText
        {
            get
            {
                return m_help_text;
            }
            set
            {
                m_help_text = value;
            }
        }

        /// <summary>
        /// The buddy control that contains a description of the button task
        /// </summary>
        public Control BuddyControl
        {
            get
            {
                return m_buddy_control;
            }
            set
            {
                m_buddy_control = value;
            }
        }

        /// <summary>
        /// The code to paint the task button
        /// </summary>
        /// <param name="pe">the arguments to the paint event</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            if (m_enabled_image != null)
            {
                Rectangle r = new Rectangle(0, 0, Width, Height);
                if (m_draw_in_color)
                {
                    if ((Control.MouseButtons & MouseButtons.Left) == MouseButtons.Left)
                        pe.Graphics.DrawImage(m_disabled_image, r);
                    else
                        pe.Graphics.DrawImage(m_enabled_image, r);
                }
                else
                {
                    if (m_disabled_image != null)
                        pe.Graphics.DrawImage(m_disabled_image, r);
                }
            }
        }
    }
}
