using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bwg.Logging ;
using Bwg.Scsi ;
using Bwg.Burner ;

namespace Bwg.CopyDisk
{
    public partial class CopySettings : Form
    {
        /// <summary>
        /// The location for temporary files
        /// </summary>
        public string TempDir;

        /// <summary>
        /// The debug level if debug messages are displayed
        /// </summary>
        public int DebugLevel;

        /// <summary>
        /// If true, display debug messages
        /// </summary>
        public bool DebugMessagesFlag;

        /// <summary>
        /// If true, display warning messages
        /// </summary>
        public bool WarningMessagesFlag;

        /// <summary>
        /// This indicates the burntype
        /// </summary>
        public Drive.BurnType BurnType;

        /// <summary>
        /// The speed to use in burning a disk
        /// </summary>
        public SpeedInfo BurnSpeedValue;

        public CopySettings(string tmp, int debug, bool debugFlag, bool warningFlag, Drive.BurnType bt, SpeedInfo [] speeds)
        {
            InitializeComponent();

            TempDir = tmp;
            DebugLevel = debug;
            DebugMessagesFlag = debugFlag;
            WarningMessagesFlag = warningFlag;
            BurnType = bt;

            //
            // Now, intiailize the controls
            //
            TemporaryDirectory.Text = TempDir;
            DebugLevelList.SelectedIndex = DebugLevel;

            if (DebugMessagesFlag)
                DebugMessages.Checked = true;
            else
                DebugMessages.Checked = false;

            if (WarningMessagesFlag)
                WarningMessages.Checked = true;
            else
                WarningMessages.Checked = false;

            switch (BurnType)
            {
                case Drive.BurnType.DontCare:
                    BurnTypeList.SelectedIndex = 0;
                    break;

                case Drive.BurnType.Session:
                    BurnTypeList.SelectedIndex = 1;
                    break;

                case Drive.BurnType.Track:
                    BurnTypeList.SelectedIndex = 2;
                    break;
            }

            if (speeds != null)
            {
                if (speeds.GetLength(0) > 0)
                {
                    for (int i = 0; i < speeds.GetLength(0); i++)
                        BurnSpeed.Items.Add(speeds[i]);

                    BurnSpeed.SelectedIndex = 0;
                    BurnSpeedValue = speeds[0];
                }
                else
                {
                    BurnSpeed.Items.Add("Maximum");
                    BurnSpeedValue = null;
                }
            }
            else
            {
                BurnSpeed.Enabled = false;
            }
        }

        private void TemporaryDirectory_TextChanged(object sender, EventArgs e)
        {
            TempDir = TemporaryDirectory.Text;
        }

        private void DebugMessages_CheckedChanged(object sender, EventArgs e)
        {
            if (DebugMessages.Checked)
                DebugMessagesFlag = true;
            else
                DebugMessagesFlag = false;
        }

        private void WarningMessages_CheckedChanged(object sender, EventArgs e)
        {
            if (WarningMessages.Checked)
                WarningMessagesFlag = true;
            else
                WarningMessagesFlag = false;
        }

        private void DebugLevelList_SelectedIndexChanged(object sender, EventArgs e)
        {
            DebugLevel = DebugLevelList.SelectedIndex;
        }

        private void BurnTypeList_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (BurnTypeList.SelectedIndex)
            {
                case 0:
                    BurnType = Drive.BurnType.DontCare;
                    break;

                case 1:
                    BurnType = Drive.BurnType.Session;
                    break;

                case 2:
                    BurnType = Drive.BurnType.Track;
                    break;
            }
        }

        private void BrowseForTempDir_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();

            dialog.RootFolder = Environment.SpecialFolder.MyComputer;
            dialog.ShowNewFolderButton = true ;
            dialog.Description = "Set a folder for storing temporary files.  Make this folder on a drive with plenty of disk space.  Copying a DVD can take up to 9 Gbytes";
            if (dialog.ShowDialog() == DialogResult.OK)
                TemporaryDirectory.Text = dialog.SelectedPath;
        }
    }
}