using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using Bwg.Burner;

namespace Bwg.CreateAudioDisk
{
    /// <summary>
    /// This class stores all of the information required to save an audio project
    /// to disk.
    /// </summary>
    [Serializable()]
    class AudioProject : IEnumerable<AudioProjectTrack>, IEnumerable
    {
        private IList<AudioProjectTrack> m_tracks;
        private byte[] m_mcn;
        private IList<CDTextEncoderDecoder.LanguageType> m_langs;
        private IList<CDTextField> m_text;

        public AudioProject(DiskBurnImage di)
        {
            m_tracks = new List<AudioProjectTrack>();
            m_text = new List<CDTextField>();
            m_langs = new List<CDTextEncoderDecoder.LanguageType>();

            if (di.HasMCN)
                m_mcn = di.MCN;

            for (int i = 0; i < di.CDTextLanguageCount; i++)
                m_langs.Add(di.GetCDTextLanguage(i));

            foreach (CDTextField tf in di.CDTextList)
                m_text.Add(tf);

            foreach (TrackBurnImage ti in di)
            {
                AudioProjectTrack at = new AudioProjectTrack(ti);
                m_tracks.Add(at);
            }
        }

        /// <summary>
        /// Return the CD text associated with this project object
        /// </summary>
        public IList<CDTextField> CDTextList
        {
            get
            {
                return m_text;
            }
        }

        /// <summary>
        /// Return the list of languages for the CD Text
        /// </summary>
        public IList<CDTextEncoderDecoder.LanguageType> CDTextLanguageList
        {
            get
            {
                return m_langs;
            }
        }

        /// <summary>
        /// Return an enumerator for iterating over all of the tracks on the
        /// disk.
        /// </summary>
        /// <returns>iterator</returns>
        public IEnumerator<AudioProjectTrack> GetEnumerator()
        {
            return m_tracks.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return m_tracks.GetEnumerator();
        }

        public bool HasMCN
        {
            get
            {
                return m_mcn != null;
            }
        }

        public byte[] MCN
        {
            get
            {
                return m_mcn;
            }
        }

        public int Count
        {
            get
            {
                return m_tracks.Count;
            }
        }
    }
}
