using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;
using Bwg.Audio;
using Bwg.Burner ;
using Bwg.Scsi ;

namespace Bwg.CreateAudioDisk
{
    [Serializable()]
    class AudioProjectTrack
    {
        private string m_filename;
        private bool m_preemphasis;
        private byte[] m_isrc;
        private IList<CDTextField> m_cdtext;
        private long m_pregap;

        public AudioProjectTrack(TrackBurnImage ti)
        {
            Debug.Assert(ti.Type == TrackBurnImage.TrackType.Audio);
            Debug.Assert(ti.Data is AudioDataSource);

            AudioDataSource src = (AudioDataSource)ti.Data;
            AudioReader rdr = src.Reader;

            m_cdtext = new List<CDTextField>();

            m_filename = rdr.Name;
            m_preemphasis = ti.PreEmphasis;
            m_pregap = ti.PreTrackGap;

            if (ti.HasIsrc)
                m_isrc = ti.ISRC;

            foreach (CDTextField field in ti.CDTextList)
                m_cdtext.Add(field);
        }

        public bool HasISRC
        {
            get
            {
                return m_isrc != null;
            }
        }

        public byte[] ISRC
        {
            get
            {
                return m_isrc;
            }
        }

        public long PreTrackGap
        {
            get
            {
                return m_pregap;
            }
        }

        public bool PreEmphasis
        {
            get
            {
                return m_preemphasis;
            }
        }

        public IList<CDTextField> CDTextList
        {
            get
            {
                return m_cdtext;
            }
        }

        public string FileName
        {
            get
            {
                return m_filename;
            }
        }
    }
}
