using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Bwg.Burner;
using Bwg.Controls;

namespace Bwg.CreateAudioDisk
{
    partial class TrackProps : FormWithSettings
    {
        private static string[] codes = new string[]
            {
                "AD", "Andorra",
                "AE", "United Arab Emirates",
                "AF", "Afghanistan",
                "AG", "Antigua and Barbuda",
                "AI", "Anguilla AI previously represented French Territory of the Afars and the Issas",
                "AL", "Albania",
                "AM", "Armenia",
                "AN", "Netherlands Antilles",
                "AO", "Angola",
                "AQ", "Antarctica defined as territory south of latitude 60S",
                "AR", "Argentina",
                "AS", "American Samoa",
                "AT", "Austria",
                "AU", "Australia including Ashmore and Cartier Islands and Coral Sea Islands",
                "AW", "Aruba",
                "AX", "land Islands",
                "AZ", "Azerbaijan",
                "BA", "Bosnia and Herzegovina",
                "BB", "Barbados",
                "BD", "Bangladesh",
                "BE", "Belgium",
                "BF", "Burkina Faso",
                "BG", "Bulgaria",
                "BH", "Bahrain",
                "BI", "Burundi",
                "BJ", "Benin",
                "BM", "Bermuda",
                "BN", "Brunei Darussalam",
                "BO", "Bolivia",
                "BR", "Brazil",
                "BS", "Bahamas",
                "BT", "Bhutan",
                "BV", "Bouvet Island",
                "BW", "Botswana",
                "BY", "Belarus",
                "BZ", "Belize",
                "CA", "Canada",
                "CC", "Cocos (Keeling) Islands",
                "CD", "Congo, the Democratic Republic of the formerly Zaire ZR",
                "CF", "Central African Republic",
                "CG", "Congo officially the Republic of the Congo",
                "CH", "Switzerland Latin: Confoederatio Helvetica",
                "CI", "Cte d'Ivoire also known as Ivory Coast",
                "CK", "Cook Islands",
                "CL", "Chile",
                "CM", "Cameroon",
                "CN", "China officially the People's Republic of China",
                "CO", "Colombia",
                "CR", "Costa Rica",
                "CS", "Serbia and Montenegro1 formerly Yugoslavia, Serbian: Srbija i Crna Gora",
                "CS", "previously represented Czechoslovakia",
                "CU", "Cuba",
                "CV", "Cape Verde",
                "CX", "Christmas Island",
                "CY", "Cyprus",
                "CZ", "Czech Republic",
                "DE", "Germany German: Deutschland",
                "DJ", "Djibouti",
                "DK", "Denmark",
                "DM", "Dominica",
                "DO", "Dominican Republic",
                "DZ", "Algeria",
                "EC", "Ecuador",
                "EE", "Estonia",
                "EG", "Egypt",
                "EH", "Western Sahara formerly Spanish Sahara",
                "ER", "Eritrea",
                "ES", "Spain including Canary Islands, Ceuta and Melilla, Spanish: Espaa",
                "ET", "Ethiopia",
                "FI", "Finland",
                "FJ", "Fiji",
                "FK", "Falkland Islands (Malvinas)",
                "FM", "Micronesia, Federated States of",
                "FO", "Faroe Islands",
                "FR", "France",
                "GA", "Gabon",
                "GB", "United Kingdom",
                "GD", "Grenada",
                "GE", "Georgia GE previously represented the Gilbert and Ellice Islands",
                "GF", "French Guiana",
                "GG", "Guernsey",
                "GH", "Ghana",
                "GI", "Gibraltar",
                "GL", "Greenland",
                "GM", "Gambia",
                "GN", "Guinea",
                "GP", "Guadeloupe",
                "GQ", "Equatorial Guinea",
                "GR", "Greece",
                "GS", "South Georgia and the South Sandwich Islands",
                "GT", "Guatemala",
                "GU", "Guam",
                "GW", "Guinea-Bissau",
                "GY", "Guyana",
                "HK", "Hong Kong officially the Hong Kong Special Administrative Region of the People's Republic of China",
                "HM", "Heard Island and McDonald Islands",
                "HN", "Honduras",
                "HR", "Croatia Croat: Hrvatska",
                "HT", "Haiti",
                "HU", "Hungary",
                "ID", "Indonesia",
                "IE", "Ireland",
                "IL", "Israel",
                "IM", "Isle of Man",
                "IN", "India",
                "IO", "British Indian Ocean Territory including Diego Garcia",
                "IQ", "Iraq",
                "IR", "Iran, Islamic Republic of",
                "IS", "Iceland",
                "IT", "Italy",
                "JE", "Jersey",
                "JM", "Jamaica",
                "JO", "Jordan",
                "JP", "Japan",
                "KE", "Kenya",
                "KG", "Kyrgyzstan",
                "KH", "Cambodia",
                "KI", "Kiribati",
                "KM", "Comoros",
                "KN", "Saint Kitts and Nevis",
                "KP", "Korea, Democratic People's Republic of i.e., North Korea",
                "KR", "Korea, Republic of i.e., South Korea",
                "KW", "Kuwait",
                "KY", "Cayman Islands",
                "KZ", "Kazakhstan",
                "LA", "Lao People's Democratic Republic",
                "LB", "Lebanon",
                "LC", "Saint Lucia",
                "LI", "Liechtenstein",
                "LK", "Sri Lanka",
                "LR", "Liberia",
                "LS", "Lesotho",
                "LT", "Lithuania",
                "LU", "Luxembourg",
                "LV", "Latvia",
                "LY", "Libyan Arab Jamahiriya",
                "MA", "Morocco",
                "MC", "Monaco",
                "MD", "Moldova, Republic of",
                "MG", "Madagascar",
                "MH", "Marshall Islands",
                "MK", "Macedonia, the former Yugoslav Republic of",
                "ML", "Mali",
                "MM", "Myanmar formerly Burma",
                "MN", "Mongolia",
                "MO", "Macao officially the Macao Special Administrative Region of the People's Republic of China",
                "MP", "Northern Mariana Islands",
                "MQ", "Martinique",
                "MR", "Mauritania",
                "MS", "Montserrat",
                "MT", "Malta",
                "MU", "Mauritius",
                "MV", "Maldives",
                "MW", "Malawi",
                "MX", "Mexico",
                "MY", "Malaysia",
                "MZ", "Mozambique",
                "NA", "Namibia",
                "NC", "New Caledonia",
                "NE", "Niger",
                "NF", "Norfolk Island",
                "NG", "Nigeria",
                "NI", "Nicaragua",
                "NL", "Netherlands",
                "NO", "Norway",
                "NP", "Nepal",
                "NR", "Nauru",
                "NU", "Niue",
                "NZ", "New Zealand",
                "OM", "Oman",
                "PA", "Panama",
                "PE", "Peru",
                "PF", "French Polynesia including Clipperton Island",
                "PG", "Papua New Guinea",
                "PH", "Philippines",
                "PK", "Pakistan",
                "PL", "Poland",
                "PM", "Saint Pierre and Miquelon",
                "PN", "Pitcairn",
                "PR", "Puerto Rico",
                "PS", "Palestinian Territory, Occupied i.e., West Bank and Gaza Strip",
                "PT", "Portugal",
                "PW", "Palau",
                "PY", "Paraguay",
                "QA", "Qatar",
                "RE", "Runion",
                "RO", "Romania",
                "RU", "Russian Federation",
                "RW", "Rwanda",
                "SA", "Saudi Arabia",
                "SB", "Solomon Islands",
                "SC", "Seychelles",
                "SD", "Sudan",
                "SE", "Sweden",
                "SG", "Singapore",
                "SH", "Saint Helena including Ascension Island and Tristan da Cunha",
                "SI", "Slovenia",
                "SJ", "Svalbard and Jan Mayen consisting of Svalbard and Jan Mayen",
                "SK", "Slovakia SK previously represented Sikkim",
                "SL", "Sierra Leone",
                "SM", "San Marino",
                "SN", "Senegal",
                "SO", "Somalia",
                "SR", "Suriname",
                "ST", "Sao Tome and Principe",
                "SV", "El Salvador",
                "SY", "Syrian Arab Republic",
                "SZ", "Swaziland",
                "TC", "Turks and Caicos Islands",
                "TD", "Chad French: Tchad",
                "TF", "French Southern Territories",
                "TG", "Togo",
                "TH", "Thailand",
                "TJ", "Tajikistan",
                "TK", "Tokelau",
                "TL", "Timor-Leste also known as East Timor",
                "TM", "Turkmenistan",
                "TN", "Tunisia",
                "TO", "Tonga",
                "TR", "Turkey",
                "TT", "Trinidad and Tobago",
                "TV", "Tuvalu",
                "TW", "Taiwan, Province of China administered by the Republic of China",
                "TZ", "Tanzania, United Republic of",
                "UA", "Ukraine",
                "UG", "Uganda",
                "UM", "United States Minor Outlying Islands consisting of Baker Island, Howland Island, Jarvis Island, Johnston Atoll, Kingman Reef, Midway Atoll, Navassa Island, Palmyra Atoll, Wake Island",
                "US", "United States",
                "UY", "Uruguay",
                "UZ", "Uzbekistan",
                "VA", "Holy See (Vatican City State)",
                "VC", "Saint Vincent and the Grenadines",
                "VE", "Venezuela",
                "VG", "Virgin Islands, British",
                "VI", "Virgin Islands, U.S.",
                "VN", "Viet Nam",
                "VU", "Vanuatu",
                "WF", "Wallis and Futuna",
                "WS", "Samoa formerly Western Samoa",
                "YE", "Yemen",
                "YT", "Mayotte",
                "ZA", "South Africa Dutch: Zuid-Afrika",
                "ZM", "Zambia",
                "ZW", "Zimbabwe"
    };

        public byte[] ISRC;

        public TrackProps(byte [] isrc, CDTextTrackProps cdtext)
        {
            string str;
            Debug.Assert(isrc == null || isrc.GetLength(0) == 12);

            InitializeComponent();

            CountryCode.Items.Clear();

            for (int i = 0; i < codes.GetLength(0); i += 2)
            {
                str = codes[i] + " - " + codes[i + 1];
                CountryCode.Items.Add(str);
            }

            if (isrc != null)
            {
                str = string.Empty;
                str += (char)isrc[0];
                str += (char)isrc[1];
                SetCountryCode(str);

                TrackOwner.Text = string.Empty;
                TrackOwner.Text += (char)isrc[2];
                TrackOwner.Text += (char)isrc[3];
                TrackOwner.Text += (char)isrc[4];

                Year.Text = string.Empty;
                Year.Text += (char)isrc[5] ;
                Year.Text += (char)isrc[6] ;

                SerialNumber.Text = string.Empty;
                SerialNumber.Text += (char)isrc[7];
                SerialNumber.Text += (char)isrc[8];
                SerialNumber.Text += (char)isrc[9];
                SerialNumber.Text += (char)isrc[10];
                SerialNumber.Text += (char)isrc[11];

                UseIsrcCode.Checked = true;
            }
            else
            {
                // Find the default country code (the last one used)
                str = GetStartupString("DefaultCountryCode", "US");
                SetCountryCode(str);

                // Find the default registrant code (the last one used)
                str = GetStartupString("DefaultRegistrantString", "000");
                TrackOwner.Text = str;

                // The current year
                int year = DateTime.Now.Year % 100;
                Year.Text = year.ToString();
                if (Year.Text.Length == 1)
                    Year.Text = "0" + Year.Text;

                // Find the last used serial # and increment
                int serial = GetStartupInteger("NextIsrcSerialNumber", 0);
                SerialNumber.Text = serial.ToString();
                SetStartupInteger("NextIsrcSerialNumber", serial + 1);
                while (SerialNumber.Text.Length < 5)
                    SerialNumber.Text = "0" + SerialNumber.Text;

                UseIsrcCode.Checked = false;
            }

            CDTextGrid.SelectedObject = cdtext;
        }

        private void ExtractISRC()
        {
            if (UseIsrcCode.Checked)
            {
                if (ISRC == null)
                    ISRC = new byte[12];

                string str = (string)CountryCode.SelectedItem;
                if (str != null && str.Length > 1)
                {
                    ISRC[0] = (byte)str[0];
                    ISRC[1] = (byte)str[1];
                }
                if (TrackOwner.Text.Length > 2)
                {
                    ISRC[2] = (byte)TrackOwner.Text[0];
                    ISRC[3] = (byte)TrackOwner.Text[1];
                    ISRC[4] = (byte)TrackOwner.Text[2];
                }

                if (Year.Text.Length > 1)
                {
                    ISRC[5] = (byte)Year.Text[0];
                    ISRC[6] = (byte)Year.Text[1];
                }

                if (SerialNumber.Text.Length > 4)
                {
                    ISRC[7] = (byte)SerialNumber.Text[0];
                    ISRC[8] = (byte)SerialNumber.Text[1];
                    ISRC[9] = (byte)SerialNumber.Text[2];
                    ISRC[10] = (byte)SerialNumber.Text[3];
                    ISRC[11] = (byte)SerialNumber.Text[4];
                }
            }
            else
                ISRC = null;
        }

        private void SetCountryCode(string code)
        {
            for (int i = 0; i < CountryCode.Items.Count; i++)
            {
                string str = (string)CountryCode.Items[i];
                if (str.Substring(0, 2) == code)
                {
                    CountryCode.SelectedIndex = i;
                    break;
                }
            }
        }

        private void UseIsrcCode_CheckedChanged(object sender, EventArgs e)
        {
            bool state = false ;
            if (UseIsrcCode.Checked)
            {
                state = true;
                ExtractISRC();
            }

            CountryCode.Enabled = state;
            Year.Enabled = state;
            SerialNumber.Enabled = state;
            TrackOwner.Enabled = state;
        }

        private void CountryCode_SelectedIndexChanged(object sender, EventArgs e)
        {
            UseIsrcCode.Checked = true;
            ExtractISRC();
        }

        private void Year_TextChanged(object sender, EventArgs e)
        {
            UseIsrcCode.Checked = true;
            ExtractISRC();
        }

        private void SerialNumber_TextChanged(object sender, EventArgs e)
        {
            int v ;
            UseIsrcCode.Checked = true;
            ExtractISRC();

            if (!Int32.TryParse(SerialNumber.Text, out v))
                v = -1;

            SetStartupInteger("NextIsrcSerialNumber", v + 1);
        }

        private void TrackOwner_TextChanged(object sender, EventArgs e)
        {
            UseIsrcCode.Checked = true;
            ExtractISRC();
            SetStartupString("DefaultRegistrantString", TrackOwner.Text);
        }

        private void RemoveCDText_Click(object sender, EventArgs e)
        {
            CDTextTrackProps props = (CDTextTrackProps)CDTextGrid.SelectedObject ;
            props.Arranger = new CDTextDataString();
            props.Composer = new CDTextDataString();
            props.Message = new CDTextDataString();
            props.Performer = new CDTextDataString();
            props.Songwriter = new CDTextDataString();
            props.Title = new CDTextDataString();
            CDTextGrid.SelectedObject = props;
        }
    }
}