//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization ;
using System.IO ;
using Bwg.Isofs ;


namespace Bwg.CreateDataDisk
{
    public partial class ImageSettingsForm : Form
    {
        /// <summary>
        /// This property returns the settings from the form
        /// </summary>
        public ImageSettings Settings
        {
            get
            {
                return m_settings;
            }
        }

        #region private member variables
        private ImageSettings m_settings;
        private bool m_joliet_state;
        private bool m_joliet_saved;
        #endregion

        /// <summary>
        /// Create a new settings form
        /// </summary>
        public ImageSettingsForm(ImageSettings settings)
        {
            m_settings = settings;
            m_joliet_saved = false;

            InitializeComponent();

            // Populate the ISO format choices
            ISOFormat.Items.Add("None");
            ISOFormat.Items.Add("ISO 9660 Level 1");
            ISOFormat.Items.Add("ISO 9660 Level 2");
            ISOFormat.Items.Add("ISO 9660 Level 3");
            ISOFormat.Items.Add("Custom Settings");

            // Populate the UDF format choices
            UDFFormat.Items.Add("None");
            UDFFormat.Items.Add("UDF 1.02");
            // UDFFormat.Items.Add("UDF 1.50");
            // UDFFormat.Items.Add("UDF 2.00");
            // UDFFormat.Items.Add("UDF 2.01");
            // UDFFormat.Items.Add("UDF 2.50");
            // UDFFormat.Items.Add("UDF 2.60");

            // Now, populate the dialog box based on the settings
            InitializeDialogFromSettings();

            LoadAddressControl.Validating += new CancelEventHandler(LoadAddressControl_Validating);
        }

        private void InitializeDialogFromSettings()
        {
            ISO9660Props.SelectedObject = m_settings.ISO9660Settings;
            JolietProps.SelectedObject = m_settings.JolietSettings;
            UDFGrid.SelectedObject = m_settings.UdfSettings;

            switch (m_settings.ISOImage)
            {
                case ImageSettings.ISOImageType.None:
                    ISOFormat.SelectedIndex = 0;
                    break;

                case ImageSettings.ISOImageType.ISO9660_Level1:
                    ISOFormat.SelectedIndex = 1;
                    break;

                case ImageSettings.ISOImageType.ISO9660_Level2:
                    ISOFormat.SelectedIndex = 2;
                    break;

                case ImageSettings.ISOImageType.ISO9660_Level3:
                    ISOFormat.SelectedIndex = 3;
                    break;

                case ImageSettings.ISOImageType.ISO9660_Custom:
                    ISOFormat.SelectedIndex = 4;
                    break;
            }

            if (m_settings.IsBootable)
                EnableBootable.Checked = true;
            else
                EnableBootable.Checked = false;

            EnableBootable_CheckedChanged(null, null);

            BootImageNameControl.Text = m_settings.BootImageName;

            switch (m_settings.Emulation)
            {
                case ImageSettings.EmulationType.NoEmulation:
                    EmulationControl.SelectedIndex = 0;
                    break;

                case ImageSettings.EmulationType.Floppy_1_2:
                    EmulationControl.SelectedIndex = 1;
                    break;

                case ImageSettings.EmulationType.Floppy_1_44:
                    EmulationControl.SelectedIndex = 2;
                    break;

                case ImageSettings.EmulationType.Floppy_2_88:
                    EmulationControl.SelectedIndex = 3;
                    break;

                case ImageSettings.EmulationType.HardDisk:
                    EmulationControl.SelectedIndex = 4;
                    break;
            }

            Platform.SelectedIndex = m_settings.BootPlatformID;
            LoadAddressControl.Text = m_settings.SegmentLoadAddr.ToString("x4");

            JolietCheckbox.Checked = m_settings.IsJoliet;
            JolietCheckbox_CheckedChanged(null, null);

            switch (m_settings.UDFImage)
            {
                case ImageSettings.UDFImageType.None:
                    UDFFormat.SelectedIndex = 0;
                    break;

                case ImageSettings.UDFImageType.UDF_1_02:
                    UDFFormat.SelectedIndex = 1;
                    break;

                case ImageSettings.UDFImageType.UDF_1_50:
                    UDFFormat.SelectedIndex = 2;
                    break;

                case ImageSettings.UDFImageType.UDF_2_00:
                    UDFFormat.SelectedIndex = 3;
                    break;

                case ImageSettings.UDFImageType.UDF_2_01:
                    UDFFormat.SelectedIndex = 4;
                    break;

                case ImageSettings.UDFImageType.UDF_2_50:
                    UDFFormat.SelectedIndex = 5;
                    break;

                case ImageSettings.UDFImageType.UDF_2_60:
                    UDFFormat.SelectedIndex = 6;
                    break;
            }
            UDFFormat_SelectedIndexChanged(null, null);
        }

        private void None()
        {
            m_settings.ISOImage = ImageSettings.ISOImageType.None;
            ISO9660Props.Enabled = false;

            m_joliet_state = JolietCheckbox.Checked;
            m_joliet_saved = true;
            JolietCheckbox.Checked = false;
            JolietCheckbox.Enabled = false;
        }

        private void Level1()
        {
            if (m_joliet_saved)
            {
                JolietCheckbox.Checked = m_joliet_state;
                JolietCheckbox.Enabled = true;
                m_joliet_saved = false;
            }

            m_settings.ISO9660Settings.AllowNonISOChars = false;
            m_settings.ISO9660Settings.ConvertCaseToUpper = true;
            m_settings.ISO9660Settings.MaximumDirectoryDepth = 0;
            m_settings.ISO9660Settings.MaximumDirectoryLength = 8;
            m_settings.ISO9660Settings.MaximumFileExtensionLength = 3;
            m_settings.ISO9660Settings.MaximumFileNameLength = 8;
            m_settings.ISO9660Settings.MaximumTotalNameLength = 0;
            m_settings.ISO9660Settings.MaximumTotalPathLength = 0;

            m_settings.ISOImage = ImageSettings.ISOImageType.ISO9660_Level1;
            ISO9660Props.SelectedObject = m_settings.ISO9660Settings;
            ISO9660Props.Enabled = true;
        }

        private void Level2()
        {
            if (m_joliet_saved)
            {
                JolietCheckbox.Checked = m_joliet_state;
                JolietCheckbox.Enabled = true;
                m_joliet_saved = false;
            }

            m_settings.ISO9660Settings.AllowNonISOChars = false;
            m_settings.ISO9660Settings.ConvertCaseToUpper = true;
            m_settings.ISO9660Settings.MaximumDirectoryDepth = 0;
            m_settings.ISO9660Settings.MaximumDirectoryLength = 207;
            m_settings.ISO9660Settings.MaximumFileExtensionLength = 0;
            m_settings.ISO9660Settings.MaximumFileNameLength = 0;
            m_settings.ISO9660Settings.MaximumTotalNameLength = 207;
            m_settings.ISO9660Settings.MaximumTotalPathLength = 0;

            m_settings.ISOImage = ImageSettings.ISOImageType.ISO9660_Level2;
            ISO9660Props.SelectedObject = m_settings.ISO9660Settings;
            ISO9660Props.Enabled = true;
        }

        private void Level3()
        {
            if (m_joliet_saved)
            {
                JolietCheckbox.Checked = m_joliet_state;
                JolietCheckbox.Enabled = true;
                m_joliet_saved = false;
            }

            m_settings.ISO9660Settings.AllowNonISOChars = false;
            m_settings.ISO9660Settings.ConvertCaseToUpper = true;
            m_settings.ISO9660Settings.MaximumDirectoryDepth = 0;
            m_settings.ISO9660Settings.MaximumDirectoryLength = 207;
            m_settings.ISO9660Settings.MaximumFileExtensionLength = 0;
            m_settings.ISO9660Settings.MaximumFileNameLength = 0;
            m_settings.ISO9660Settings.MaximumTotalNameLength = 207;
            m_settings.ISO9660Settings.MaximumTotalPathLength = 0;

            m_settings.ISOImage = ImageSettings.ISOImageType.ISO9660_Level3;
            ISO9660Props.SelectedObject = m_settings.ISO9660Settings;
            ISO9660Props.Enabled = true;
        }

        private void Custom()
        {
            m_settings.ISO9660Settings.AllowNonISOChars = true;
            m_settings.ISO9660Settings.ConvertCaseToUpper = false;
            m_settings.ISO9660Settings.MaximumDirectoryDepth = 0;
            m_settings.ISO9660Settings.MaximumDirectoryLength = 207;
            m_settings.ISO9660Settings.MaximumFileExtensionLength = 0;
            m_settings.ISO9660Settings.MaximumFileNameLength = 0;
            m_settings.ISO9660Settings.MaximumTotalNameLength = 207;
            m_settings.ISO9660Settings.MaximumTotalPathLength = 0;

            m_settings.ISOImage = ImageSettings.ISOImageType.ISO9660_Custom;
            ISO9660Props.SelectedObject = m_settings.ISO9660Settings;
            ISO9660Props.Enabled = true;
        }

        private void Formats_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (ISOFormat.SelectedIndex)
            {
                case 0: None(); break;
                case 1: Level1(); break;
                case 2: Level2(); break;
                case 3: Level3(); break;
                case 4: Custom(); break;
            }
        }

        private void OKButton_Click(object sender, EventArgs e)
        {
            if (m_settings.ISOImage == ImageSettings.ISOImageType.None &&
                m_settings.UDFImage == ImageSettings.UDFImageType.None)
            {
                MessageBox.Show("Neither UDF or ISO image type is selected.  You must select one or the other.");
                return;
            }

            DialogResult = DialogResult.OK;
            Close();
        }

        private void BrowseForImage_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();

            dialog.AddExtension = true;
            dialog.CheckFileExists = true;
            dialog.CheckPathExists = true;
            dialog.DefaultExt = "ima";
            dialog.Multiselect = false;
            dialog.Filter = "Disk Image Files (*.ima)|*.ima|All files (*.*)|*.*";
            dialog.RestoreDirectory = true;

            if (dialog.ShowDialog() == DialogResult.OK)
                BootImageNameControl.Text = dialog.FileName;
        }

        private void JolietCheckbox_CheckedChanged(object sender, EventArgs e)
        {
            if (JolietCheckbox.Checked)
            {
                m_settings.IsJoliet = true;
                JolietProps.Enabled = true;
            }
            else
            {
                m_settings.IsJoliet = false;
                JolietProps.Enabled = false;
            }
        }

        private void UDFFormat_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (UDFFormat.SelectedIndex)
            {
                case 0:
                    m_settings.UDFImage = ImageSettings.UDFImageType.None;
                    break;

                case 1:
                    m_settings.UDFImage = ImageSettings.UDFImageType.UDF_1_02;
                    break;

                case 2:
                    m_settings.UDFImage = ImageSettings.UDFImageType.UDF_1_50;
                    break;

                case 3:
                    m_settings.UDFImage = ImageSettings.UDFImageType.UDF_2_00;
                    break;

                case 4:
                    m_settings.UDFImage = ImageSettings.UDFImageType.UDF_2_01;
                    break;

                case 5:
                    m_settings.UDFImage = ImageSettings.UDFImageType.UDF_2_50;
                    break;

                case 6:
                    m_settings.UDFImage = ImageSettings.UDFImageType.UDF_2_60;
                    break;
            }
        }

        private void EnableBootable_CheckedChanged(object sender, EventArgs e)
        {
            BootImageNameControl.Enabled = EnableBootable.Checked;
            EmulationControl.Enabled = EnableBootable.Checked;
            LoadAddressControl.Enabled = EnableBootable.Checked;
            Platform.Enabled = EnableBootable.Checked;
        }

        private void CopyFromJoliet_Click(object sender, EventArgs e)
        {
            m_settings.ISO9660Settings.AbstractFileIdentifier = m_settings.JolietSettings.AbstractFileIdentifier.ToUpper();
            m_settings.ISO9660Settings.ApplicationIdentifier = m_settings.JolietSettings.ApplicationIdentifier.ToUpper();
            m_settings.ISO9660Settings.BibliographicFileIdentifier = m_settings.JolietSettings.BibliographicFileIdentifier.ToUpper();
            m_settings.ISO9660Settings.CopyrightFileIdentifier = m_settings.JolietSettings.CopyrightFileIdentifier.ToUpper();
            m_settings.ISO9660Settings.Preparer = m_settings.JolietSettings.Preparer.ToUpper();
            m_settings.ISO9660Settings.Publisher = m_settings.JolietSettings.Publisher.ToUpper();
            m_settings.ISO9660Settings.SystemIdentifier = m_settings.JolietSettings.SystemIdentifier.ToUpper();
            m_settings.ISO9660Settings.VolumeIdentifier = m_settings.JolietSettings.VolumeIdentifier.ToUpper();
            m_settings.ISO9660Settings.VolumeSetIdentifier = m_settings.JolietSettings.VolumeSetIdentifier.ToUpper();

            m_settings.ISO9660Settings.CreationDateTime = m_settings.JolietSettings.CreationDateTime;
            m_settings.ISO9660Settings.EffectiveDateTime = m_settings.JolietSettings.EffectiveDateTime;
            m_settings.ISO9660Settings.ExpirationDateTime = m_settings.JolietSettings.ExpirationDateTime;
            m_settings.ISO9660Settings.ModificationDateTime = m_settings.JolietSettings.ModificationDateTime;

            ISO9660Props.SelectedObject = m_settings.ISO9660Settings;
        }

        private void EmulationControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (EmulationControl.SelectedIndex)
            {
                case 0:
                    m_settings.Emulation = ImageSettings.EmulationType.NoEmulation;
                    break;

                case 1:
                    m_settings.Emulation = ImageSettings.EmulationType.Floppy_1_2;
                    break;

                case 2:
                    m_settings.Emulation = ImageSettings.EmulationType.Floppy_1_44;
                    break;

                case 3:
                    m_settings.Emulation = ImageSettings.EmulationType.Floppy_2_88;
                    break;

                case 4:
                    m_settings.Emulation = ImageSettings.EmulationType.HardDisk;
                    break;

            }
        }

        private void Platform_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_settings.BootPlatformID = (byte)Platform.SelectedIndex;
        }

        private void BootImageNameControl_TextChanged(object sender, EventArgs e)
        {
            m_settings.BootImageName = BootImageNameControl.Text;
        }

        private void LoadAddressControl_Validating(object sender, CancelEventArgs e)
        {
            try
            {
                int v = Int32.Parse(LoadAddressControl.Text, NumberStyles.HexNumber);
                if (v > UInt16.MaxValue)
                {
                    MessageBox.Show("Invalid load address, greater than " + UInt16.MaxValue.ToString(), "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    e.Cancel = true;
                }
            }
            catch (Exception)
            {
                MessageBox.Show("Invalid hex number", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                e.Cancel = true;
            }
        }
    }
}