//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Database.Freedb
{
    /// <summary>
    /// This class represents a set of matches from the freedb.org database
    /// </summary>
    public class FreeDBQueryRequest : FreeDBRequest
    {
        #region public types
        public enum StatusType
        {
            NotProcessed,
            ExactMatch,
            FuzzyMatch,
            NoMatch
        } ;
        #endregion

        #region private member variables
        /// <summary>
        /// The result status of the query
        /// </summary>
        private StatusType m_status;

        /// <summary>
        /// This is the array of ablums returned as the result of a freedb.org
        /// query
        /// </summary>
        private FreeDBAlbumInfo[] m_albums;

        /// <summary>
        /// This is the request information.  It contains the track count and length for each
        /// track in the request.
        /// </summary>
        private FreeDBAlbumInfo m_request;
        #endregion

        #region constructor
        public FreeDBQueryRequest(FreeDBAlbumInfo info)
        {
            m_albums = new FreeDBAlbumInfo[0];
            m_status = StatusType.NotProcessed;
            m_request = info ;
        }
        #endregion

        #region public properties
        /// <summary>
        /// This property returns the total number of albums in the set that match
        /// the specific query issued.
        /// </summary>
        public int MatchCount
        {
            get
            {
                return m_albums.GetLength(0);
            }
        }

        /// <summary>
        /// The number of tracks in the album
        /// </summary>
        public int TrackCount
        {
            get
            {
                return m_request.TrackCount;
            }
        }

        /// <summary>
        /// The status of the request
        /// </summary>
        public StatusType Status
        {
            get
            {
                return m_status;
            }
            internal set
            {
                m_status = value;
            }
        }

        /// <summary>
        /// The request to send to freedb.org
        /// </summary>
        public FreeDBAlbumInfo RequestInfo
        {
            get
            {
                return m_request;
            }
        }

        #endregion

        #region public methods

        /// <summary>
        /// This is the array accessor for accessing the various albums in the
        /// response set.
        /// </summary>
        /// <param name="index">the index of the entry to retreive</param>
        /// <returns>the specific match requested</returns>
        public FreeDBAlbumInfo this[int index]
        {
            get
            {
                return m_albums[index];
            }
        }

        /// <summary>
        /// This method gets an album if it exists, or creates one if it does not
        /// exist.
        /// </summary>
        /// <param name="index">the index of the match to get</param>
        /// <returns>the album info for the index</returns>
        public FreeDBAlbumInfo GetCreateAlbum(int index)
        {
            if (index >= m_albums.GetLength(0))
            {
                Debug.Assert(index == m_albums.GetLength(0));
                Array.Resize(ref m_albums, index + 1);

                m_albums[index] = (FreeDBAlbumInfo)m_request.Clone();
                m_albums[index].FromFreedb = true;
            }

            return m_albums[index];
        }
        #endregion
    }
}
