//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Bwg.Database.Freedb
{
    public partial class FreeDBSetupForm : Form
    {
        /// <summary>
        /// The site to connect to (e.g. srvr = new FreeDBServer(form.SelectedSite)
        /// </summary>
        public FreeDBSite SelectedSite;

        /// <summary>
        /// The name of the application
        /// </summary>
        private string m_app_name;

        /// <summary>
        /// The version of the application
        /// </summary>
        private string m_app_version;

        public FreeDBSetupForm(FreeDBSite site, string app, string ver)
        {
            m_app_name = app;
            m_app_version = ver;

            InitializeComponent();

            if (site != null)
            {
                HostName.Text = site.HostName ;
                PortNumber.Text = site.Port.ToString();
                if (site.Protocol.ToLower() == FreeDBSite.CddbProtocolName)
                    CddbButton.Checked = true;
                else
                {
                    HttpButton.Checked = true;
                    CgiPath.Text = site.HttpCgiPath;
                }
            }

            if (FreeDBServer.BehindFirewall)
                Firewall.Checked = true;
            else
                Firewall.Checked = false;

            SiteList.DoubleClick += new EventHandler(SiteList_DoubleClick);
            SelectedSite = null;
            CreateSelectedSite();
        }

        private void CreateSelectedSite()
        {
            if (SelectedSite == null)
                SelectedSite = new FreeDBSite("freedb.freedb.org", 8880, FreeDBSite.CddbProtocolName);

            SelectedSite.HostName = HostName.Text;
            Int32.TryParse(PortNumber.Text, out SelectedSite.Port);
            if (CddbButton.Checked)
                SelectedSite.Protocol = FreeDBSite.CddbProtocolName;
            else
            {
                SelectedSite.Protocol = FreeDBSite.HttpProtocolName;
                SelectedSite.HttpCgiPath = CgiPath.Text;
            }
        }

        private void PopulateSites()
        {
            FreeDBUserInfo info = new FreeDBUserInfo(m_app_name, m_app_version);
            IList<FreeDBSite> sites = FreeDBServer.GetSites(info, FreeDBServer.TransportTypes.TranportTypeCddbp | FreeDBServer.TransportTypes.TransportTypeHttp);

            SiteList.Items.Clear();

            if (sites != null)
            {
                foreach (FreeDBSite site in sites)
                {
                    ListViewItem item = new ListViewItem(site.HostName);
                    item.SubItems.Add(site.Port.ToString());
                    item.SubItems.Add(site.Protocol);
                    item.SubItems.Add(site.Location);

                    item.Tag = site;
                    SiteList.Items.Add(item);
                }
            }
        }

        private void SiteList_DoubleClick(object sender, EventArgs args)
        {
            FreeDBSite s = (FreeDBSite)SiteList.SelectedItems[0].Tag;
            HostName.Text = s.HostName;
            PortNumber.Text = s.Port.ToString();
            if (s.Protocol.ToLower() == FreeDBSite.CddbProtocolName)
            {
                CddbButton.Checked = true;
            }
            else
            {
                HttpButton.Checked = true;
                CgiPath.Text = s.HttpCgiPath;
            }
        }

        private void CddbButton_CheckedChanged(object sender, EventArgs e)
        {
            CreateSelectedSite();
            CgiPath.Enabled = false;
        }

        private void HttpButton_CheckedChanged(object sender, EventArgs e)
        {
            CreateSelectedSite();
            CgiPath.Enabled = true;
        }

        private void CgiPath_TextChanged(object sender, EventArgs e)
        {
            CreateSelectedSite();
        }

        private void PortNumber_TextChanged(object sender, EventArgs e)
        {
            CreateSelectedSite();
        }

        private void HostName_TextChanged(object sender, EventArgs e)
        {
            CreateSelectedSite();
        }

        private void Sites_Click(object sender, EventArgs e)
        {
            PopulateSites();
        }

        private void Firewall_CheckedChanged(object sender, EventArgs e)
        {
            FreeDBServer.BehindFirewall = Firewall.Checked;
        }
    }
}