//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Database.Freedb
{
    public class FreeDBTrackInfo
    {
        #region private member variables
        /// <summary>
        /// The title of this track
        /// </summary>
        private string m_title;

        /// <summary>
        /// The artiest for this track, if empty it is the disc artist
        /// </summary>
        private string m_artist;
        #endregion

        #region public properties

        /// <summary>
        /// If true, this track (song) has a specifc artist and the artist associated
        /// with this track should be used instead of any artist associated with the
        /// album
        /// </summary>
        public bool HasArtist
        {
            get
            {
                return m_artist.Length > 0;
            }
        }

        /// <summary>
        /// This string property is the title of the track
        /// </summary>
        public string Title
        {
            get
            {
                return m_title;
            }
            set
            {
                m_title = value;
            }
        }

        /// <summary>
        /// This string property is the artist that performed the track.  If this
        /// string is empty, the artist is the artist associated with the album
        /// </summary>
        public string Artist
        {
            get
            {
                return m_artist;
            }
            set
            {
                m_artist = value;
            }
        }

        /// <summary>
        /// Then number of minutes in the start location of this track
        /// </summary>
        public readonly byte StartMinute;

        /// <summary>
        /// The number of seconds in the start location of this track
        /// </summary>
        public readonly byte StartSecond;

        /// <summary>
        /// The number of frames in the start location of this track
        /// </summary>
        public readonly byte StartFrame;

        /// <summary>
        /// The length of the track in minutes
        /// </summary>
        public readonly byte LengthMinute;

        /// <summary>
        /// The length of the track in seconds
        /// </summary>
        public readonly byte LengthSecond;

        /// <summary>
        /// The length of the track in frames
        /// </summary>
        public readonly byte LengthFrame;

        #endregion

        #region constructor
        public FreeDBTrackInfo(byte m, byte s, byte f, byte ml, byte sl, byte fl)
        {
            StartMinute = m;
            StartSecond = s;
            StartFrame = f;

            LengthMinute = ml;
            LengthSecond = sl;
            LengthFrame = fl;

            m_title = string.Empty;
            m_artist = string.Empty;
        }
        #endregion
    }
}
