//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;

namespace Bwg.Database.Freedb
{
    public abstract class FreeDBTransport : IDisposable
    {
        /// <summary>
        /// The status of the request to the transport
        /// </summary>
        public enum RequestStatus
        {
            /// <summary>
            /// The request was delivered sucessfully and the response was parsed
            /// correctly.
            /// </summary>
            Success,

            /// <summary>
            /// Cannot connect the remote host
            /// </summary>
            CannotConnect,

            /// <summary>
            /// There was an error sending data to the server, or receiving data from the
            /// server
            /// </summary>
            SocketError,

            /// <summary>
            /// An invalid response was returned from the host
            /// </summary>
            InvalidReponse,

            /// <summary>
            /// login to the site was not accepted
            /// </summary>
            CannotLogin,
        } ;

        /// <summary>
        /// This method is called just before a set of requests to the server are going to be
        /// issued.  For some transports, this is a signal to make a connection to the server.  Other
        /// transports ignore this call
        /// </summary>
        /// <returns></returns>
        public abstract bool Start();

        /// <summary>
        /// This method is called after a set of requests have been processed and there is no more work
        /// to be done.  For some transports, this is a signal to close the connection to the server.  Other
        /// transports ignore this call.
        /// </summary>
        /// <returns></returns>
        public abstract bool Finish();

        /// <summary>
        /// This method sets the protol version that the transport should use.
        /// </summary>
        /// <param name="p">the protocol version</param>
        /// <returns></returns>
        public abstract bool SetProtocol(int p);

        /// <summary>
        /// This method will send a command to the CDDB server and retreive the response from the
        /// server.  The response is either a single line response, or a set of lines that are terminated
        /// with a '.' on a single line.  The command is the CDDB type command.  It is the responsibility
        /// of the transport to quote it if necessary.
        /// 
        /// The reponse may be a string, or it may be an array of strings.  The calling code must check the
        /// type or the return code to understand which it will be.
        /// </summary>
        /// <param name="cmd">the command string to send</param>
        public abstract RequestStatus SendCommandGetResponse(string cmd, out int code, out object response);

        public virtual void Dispose()
        {
        }

        protected int GetResponseCode(ref string str)
        {
            string num = string.Empty;
            int n = 0;
            int ret = -1 ;

            while (n < str.Length && Char.IsDigit(str[n]))
                num += str[n++];

            if (num.Length > 0)
            {
                try
                {
                    ret = Int32.Parse(num);
                    str = str.Substring(num.Length).Trim();
                }
                catch (Exception)
                {
                }
            }

            return ret;
        }
    }
}
