//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Net ;
using System.Net.Sockets ;
using System.Threading;

namespace Bwg.Database.Freedb
{
    class FreeDBTransportCddb : FreeDBTransport
    {
        /// <summary>
        /// The socket used to talk to the host
        /// </summary>
        private Socket m_socket;

        /// <summary>
        /// The host name of the host serving the CDDBP protocol.
        /// </summary>
        private string m_host;

        /// <summary>
        /// This is the port number on the host serving the CDDBP protocol.
        /// </summary>
        private int m_port;

        /// <summary>
        /// This contains all of the information about the user
        /// </summary>
        private FreeDBUserInfo m_user_info ;

        public FreeDBTransportCddb(FreeDBUserInfo info, string hostname, int port)
        {
            m_socket = null;
            m_user_info = info;
            m_host = hostname;
            m_port = port;
        }

        public override bool Start()
        {
            if (m_socket == null)
            {
                m_socket = GetSocket();
                if (m_socket == null)
                    return false;

                if (!Login())
                {
                    m_socket.Close();
                    m_socket = null;
                    return false;
                }
            }

            return true;
        }

        public override bool Finish()
        {
            if (m_socket != null)
            {
                m_socket.Close();
                m_socket = null;
            }

            return true;
        }

        public override bool SetProtocol(int p)
        {
            object response;
            int code;

            string cmd = "proto " + p.ToString();
            SendCommandGetResponse(cmd, out code, out response);

            return code == 201;
        }

        public override RequestStatus SendCommandGetResponse(string cmd, out int code, out object response)
        {
            code = -1;
            response = null;

            if (m_socket == null)
            {
                m_socket = GetSocket();
                if (m_socket == null)
                    return RequestStatus.CannotConnect;

                if (!Login())
                    return RequestStatus.CannotLogin;
            }

            return MySendCommandGetResponse(cmd, out code, out response);
        }

        public override void Dispose()
        {
            if (m_socket != null)
            {
                m_socket.Close();
                m_socket = null;
            }
        }

        #region private methods
        private string ReadLine(Socket s)
        {
            string line = string.Empty;
            byte[] b = new byte[1];

            if (s == null)
                s = m_socket;

            while (true)
            {
                s.Receive(b);
                string ch = Encoding.UTF8.GetString(b);
                if (ch == "\n")
                    break;

                if (ch != "\r" && ch != "\n")
                    line += ch;
            }

            return line;
        }

        private RequestStatus MySendCommandGetResponse(string cmd, out int code, out object response)
        {
            code = -1 ;
            response = null ;

            // Send the command to the server
            try
            {
                cmd += "\r\n";
                m_socket.Send(Encoding.UTF8.GetBytes(cmd));
            }
            catch (Exception)
            {
                return RequestStatus.SocketError;
            }

            // Get the first line from the socket
            string line = ReadLine(null);

            if (line.Contains("until terminating"))
            {
                string[] lines = new string[0];
                code = GetResponseCode(ref line);

                while (true)
                {
                    line = ReadLine(null);
                    if (line == ".")
                        break;

                    Array.Resize(ref lines, lines.GetLength(0) + 1);
                    lines[lines.GetLength(0) - 1] = line;
                }

                response = lines;
            }
            else
            {
                code = GetResponseCode(ref line);
                response = line;
            }

            return RequestStatus.Success;
        }

        private bool Login()
        {
            int code;
            object response;
            string cmd;

            cmd = "cddb hello " + m_user_info.UserName + " " + m_user_info.UserHost + " ";
            cmd += m_user_info.Client + " " + m_user_info.ClientVersion ;
            MySendCommandGetResponse(cmd, out code, out response);

            return code == 200;
        }

        private Socket GetSocket()
        {
            IPHostEntry entry ;

            try
            {
                entry = Dns.GetHostEntry(m_host);
            }
            catch (Exception)
            {
                return null;
            }

            Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP);

            foreach (IPAddress addr in entry.AddressList)
            {
                try
                {
                    s.Connect(addr, m_port);
                    ReadLine(s);
                    return s;
                }
                catch (Exception)
                {
                    continue;
                }
            }

            return null;
        }
        #endregion
    }
}
