using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bwg.Controls;
using Bwg.DvdMaster;
using Bwg.DvdReader;

namespace Bwg.DvdExplorer
{
    public partial class DvdExplorerForm : BwgBurnBaseForm
    {
        private DvdDisc m_disc;

        private TreeNode m_vmg_ptt_srpt_node;
        private TreeNode m_vmgm_pgci_ut_node ;

        public DvdExplorerForm()
        {
            InitializeComponent();
            DvdTreeView.AfterSelect += new TreeViewEventHandler(DvdTreeView_AfterSelect);
        }

        private void DvdTreeView_AfterSelect(object senter, TreeViewEventArgs args)
        {
            UpdatePropertyGrid(args.Node);
        }

        private bool TextIsNumber(string str)
        {
            int res;
            return int.TryParse(str, out res);
        }

        private void UpdatePropertyGrid(TreeNode node)
        {
            if (node.Text == "VMGM_MAT")
                DvdPropertyGrid.SelectedObject = m_disc.VideoManagerInformationManagement;
            else if (node.Text == "VMG_PTT_SRPT")
                DvdPropertyGrid.SelectedObject = m_disc.VideoManagerPartOfTitleSearch;
            else if (node.Text == "VMGM_PGCI_UT")
                DvdPropertyGrid.SelectedObject = m_disc.VideoManagerProgramChainUnits;
            else if (TextIsNumber(node.Text))
            {
                int num = int.Parse(node.Text);

                if (TextIsNumber(node.Parent.Text))
                {
                    int num2 = int.Parse(node.Parent.Text);
                    if (node.Parent.Parent.Text == "VMGM_PGCI_UT")
                    {
                        VMGM_PGCI_UT_Language_Unit lu = m_disc.VideoManagerProgramChainUnits[num2 - 1];
                        DvdPropertyGrid.SelectedObject = lu[num - 1];
                    }
                }
                else
                {
                    if (node.Parent.Text == "VMG_PTT_SRPT")
                        DvdPropertyGrid.SelectedObject = m_disc.VideoManagerPartOfTitleSearch[num - 1];
                    else if (node.Parent.Text == "VMGM_PGCI_UT")
                        DvdPropertyGrid.SelectedObject = m_disc.VideoManagerProgramChainUnits[num - 1];
                }
            }
            else
            {
                DvdPropertyGrid.SelectedObject = null;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            dialog.Description = "Select the root directory of the DVD";
            dialog.ShowNewFolderButton = false;
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                PathLabel.Text = dialog.SelectedPath;
                DvdPropertyGrid.SelectedObject = null;
                DvdTreeView.Nodes.Clear();

                m_disc = new DvdDisc();
                try
                {
                    Reader rdr = new Reader(dialog.SelectedPath, m_disc);
                    if (rdr.ReadDiscContents())
                    {
                        PopulateDvdTree();
                        DvdTreeView.ExpandAll();
                    }
                }
                catch (BadDataStructureException)
                {
                }
                catch (Exception)
                {
                }
            }
        }

        private void PopulateSearchTitles()
        {
            for(int i = 1 ; i <= m_disc.VideoManagerPartOfTitleSearch.NumberOfTitles ; i++)
                m_vmg_ptt_srpt_node.Nodes.Add(i.ToString()) ;
        }

        private void PopulateProgramChainUnits()
        {
            for (int i = 1; i <= m_disc.VideoManagerProgramChainUnits.NumberOfLanguageUnits; i++)
            {
                TreeNode node = new TreeNode(i.ToString());
                m_vmgm_pgci_ut_node.Nodes.Add(node);
                VMGM_PGCI_UT_Language_Unit lu = m_disc.VideoManagerProgramChainUnits[i - 1] ;
                for (int j = 1; j <= lu.NumberOfMenus; j++)
                    node.Nodes.Add(j.ToString());
            }
        }

        private void PopulateDvdTree()
        {
            TreeNode dvdnode = new TreeNode(PathLabel.Text);
            DvdTreeView.Nodes.Add(dvdnode);

            TreeNode video_ts_node = new TreeNode("VIDEO_TS.IFO");
            dvdnode.Nodes.Add(video_ts_node);

            video_ts_node.Nodes.Add("VMGM_MAT");
            video_ts_node.Nodes.Add("First Play PGC");

            m_vmg_ptt_srpt_node = new TreeNode("VMG_PTT_SRPT");
            video_ts_node.Nodes.Add(m_vmg_ptt_srpt_node);
            PopulateSearchTitles();

            m_vmgm_pgci_ut_node = new TreeNode("VMGM_PGCI_UT") ;
            video_ts_node.Nodes.Add(m_vmgm_pgci_ut_node);
            PopulateProgramChainUnits();

            video_ts_node.Nodes.Add("LU");
            video_ts_node.Nodes.Add("VMG_PTL_MAIT");
            video_ts_node.Nodes.Add("VMG_VTS_ATRT");
            video_ts_node.Nodes.Add("VMG_TXTDT_MG");
            video_ts_node.Nodes.Add("VMGM_C_ADT");
            video_ts_node.Nodes.Add("VMGM_VOBU_ADMAP");
        }
    }
}