using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.IO;
using System.Security;
using Microsoft.Win32;

namespace Bwg.Hardware
{
    public class HardwareManager : IEnumerable<HardwareDevice>, IEnumerable
    {
        #region private member variables
        private IList<HardwareDevice> m_devices;
        #endregion

        public HardwareManager()
        {
            m_devices = new List<HardwareDevice>();
        }

        /// <summary>
        /// Return an enumerator for iterating over all of the tracks on the
        /// disk.
        /// </summary>
        /// <returns>iterator</returns>
        public IEnumerator<HardwareDevice> GetEnumerator()
        {
            return m_devices.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return m_devices.GetEnumerator();
        }

        public bool OpenMachine(string keyname)
        {
            if (keyname == string.Empty)
                keyname = "System\\CurrentControlSet\\Enum";

            RegistryKey key = Registry.LocalMachine.OpenSubKey(keyname);
            if (key == null)
                return false ;

            CheckForHardware(key) ;
            key.Close() ;
            return true ;
        }

        public HardwareDevice FindDevice(string classname, string name)
        {
            foreach (HardwareDevice dev in m_devices)
            {
                if (dev.Class == classname)
                {
                    if (dev.Name == name)
                        return dev;

                    if (dev.Name.Length > name.Length && dev.Name.Substring(0, name.Length) == name)
                        return dev;
                }
            }
            return null;
        }

        private static bool FindClassString(string s1)
        {
            return s1 == "Class";
        }

        private static bool FindClassGUIDString(string s1)
        {
            return s1 == "ClassGUID";
        }

        private void CreateDevice(RegistryKey key)
        {
            string keyname = key.ToString();
            string onekey = Path.GetFileName(keyname);
            HardwareDevice dev = new HardwareDevice(onekey);
            object value ;

            value = key.GetValue("Class");
            if (value != null && value is string)
                dev.Class = (string)value;

            value = key.GetValue("ClassGUID");
            if (value != null && value is string)
                dev.ClassGUID = (string)value;

            value = key.GetValue("DeviceDesc");
            if (value != null && value is string)
                dev.Description = (string)value;

            value = key.GetValue("FriendlyName");
            if (value != null && value is string)
                dev.Name = (string)value;

            value = key.GetValue("HardwareID");
            if (value != null)
            {
                if (value is string)
                    dev.AddHardware((string)value);
                else if (value is string[])
                {
                    foreach (string str in (string[])value)
                        dev.AddHardware(str);
                }
            }

            m_devices.Add(dev);
        }

        private void CheckForHardware(RegistryKey key)
        {
            string[] values = key.GetValueNames();
            if (Array.Find(values, FindClassString) != null && Array.Find(values, FindClassGUIDString) != null)
            {
                // This is a device
                CreateDevice(key);
            }
            else
            {
                values = key.GetSubKeyNames();
                foreach (string str in values)
                {
                    try
                    {
                        RegistryKey subkey = key.OpenSubKey(str);
                        if (subkey != null)
                        {
                            CheckForHardware(subkey);
                            subkey.Close();
                        }
                    }
                    catch (SecurityException)
                    {
                    }
                }
            }
        }
    }
}
