//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel ;
using System.Diagnostics;
using Bwg.Logging ;

namespace Bwg.Isofs
{
    abstract class CreatorTask
    {
        public long BlockAddress;
        public long BlockSize;

        public ProgressChangedEventHandler ProgressChanged;

        public CreatorTask()
        {
            BlockAddress = -1;
            BlockSize = 0;
            ProgressChanged = null; 
        }

        public ExtentAd ExtentAd
        {
            get
            {
                return new ExtentAd(BlockAddress, BlockSize * 2048);
            }
        }

        public abstract bool UpdateLayout(DiskImageCreator creator, DiskInfo info);
        public abstract bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm);

        protected long RoundToNearestBlock(long value)
        {
            long blocks = value / 2048;
            if ((value % 2048) != 0)
                blocks++;

            return blocks * 2048;
        }

        protected int RoundToNearestBlock(int value)
        {
            return (int)RoundToNearestBlock((long)value);
        }

        static internal Encoding GetEncoding(DiskImageElement.NameType type)
        {
            Encoding ret = Encoding.ASCII;

            switch (type)
            {
                case DiskImageElement.NameType.ISO9660:
                    ret = Encoding.ASCII;
                    break;

                case DiskImageElement.NameType.Joliet:
                    ret = Encoding.BigEndianUnicode;
                    break;

                case DiskImageElement.NameType.UDF:
                    ret = Encoding.BigEndianUnicode;
                    break;

                default:
                    Debug.Assert(false);
                    break;
            }

            return ret;
        }

        static internal DiskImageDirectory.DirectoryLocationType NameTypeToDirLocationType(DiskImageElement.NameType type)
        {
            DiskImageDirectory.DirectoryLocationType dirtype = DiskImageDirectory.DirectoryLocationType.ISO9660;

            switch (type)
            {
                case DiskImageElement.NameType.ISO9660:
                    dirtype = DiskImageDirectory.DirectoryLocationType.ISO9660;
                    break;

                case DiskImageElement.NameType.Joliet:
                    dirtype = DiskImageDirectory.DirectoryLocationType.Joliet;
                    break;

                case DiskImageElement.NameType.UDF:
                    dirtype = DiskImageDirectory.DirectoryLocationType.UdfFileIdentifierDescriptor;
                    break;

                default:
                    Debug.Assert(false);
                    break;
            }

            return dirtype;
        }
    }
}
