//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;

namespace Bwg.Isofs
{
    /// <summary>
    /// This class is the argument to the event handler that is called when
    /// a directory within the DiskImageCreator changes.
    /// </summary>
    public class DiskImageDirectoryChangedArgs : EventArgs
    {
        /// <summary>
        /// The type of change that has occurred
        /// </summary>
        public enum ChangeType
        {
            /// <summary>
            /// An element was added to the directory
            /// </summary>
            AddElement,
            
            /// <summary>
            /// An element was removed from a directory
            /// </summary>
            RemoveElement,

            /// <summary>
            /// All of the elemnets were removed from a directory
            /// </summary>
            RemoveAllElements
        } ;

        /// <summary>
        /// The type of change that occurred
        /// </summary>
        public ChangeType Change;

        /// <summary>
        /// The element involved in the change, or null for RemoveAllElements
        /// </summary>
        public DiskImageElement Element;

        /// <summary>
        /// Create an image directory changed arg
        /// </summary>
        /// <param name="t">the type of change</param>
        /// <param name="e">the element in the change</param>
        public DiskImageDirectoryChangedArgs(ChangeType t, DiskImageElement e)
        {
            Change = t;
            Element = e;
        }
    }
}
