//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.IO ;
using System.Diagnostics;
using Bwg.Logging;

namespace Bwg.Isofs
{
    class ElToritoBootImage : CreatorTask
    {
        #region private data members
        /// <summary>
        /// The file information for the boot image
        /// </summary>
        private FileInfo m_info;

        /// <summary>
        /// The length of the image in sectors
        /// </summary>
        private int m_length;
        #endregion

        #region public properties
        public int LengthInBlocks
        {
            get
            {
                return m_length;
            }
        }
        #endregion

        #region public member methods
        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            bool ret = true ;

            try
            {
                m_info = new FileInfo(creator.Settings.BootImageName);

                // Be sure we don't have overflow problems
                Debug.Assert(m_info.Length / 2048 < Int32.MaxValue);

                m_length = (int)(m_info.Length / 2048);
                if ((m_info.Length % 2048) != 0)
                    m_length++;

                info.CurrentBlock += m_length;
            }
            catch (Exception ex)
            {
                string str;

                str = "cannot open file '" + creator.Settings.BootImageName + "' - " + ex.Message;
                creator.GetLogger().LogMessage(new UserMessage(UserMessage.Category.Error, 0, str));
                ret = false;
            }

            return ret;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            bool ret = true;
            long pos = strm.Position;
            long endpos = pos + m_length * 2048;

            try
            {
                byte[] block = new byte[2048];
                FileStream fstr = new FileStream(creator.Settings.BootImageName, FileMode.Open, FileAccess.Read);

                while (true)
                {
                    int cnt;

                    cnt = fstr.Read(block, 0, 2048);
                    if (cnt == 0)
                        break;

                    strm.Write(block);
                }

                strm.WriteZeros((uint)(endpos - strm.Position));
            }
            catch (Exception ex)
            {
                string str;

                str = "cannot open file '" + creator.Settings.BootImageName + "' - " + ex.Message;
                creator.GetLogger().LogMessage(new UserMessage(UserMessage.Category.Error, 0, str));
                ret = false;
            }

            return ret;
        }
        #endregion
    }
}
