//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;

namespace Bwg.Isofs
{
    class FileEntryDescriptor : InformationControlBlockTask
    {
        private DiskImageFile m_file ;

        public FileEntryDescriptor(DiskImageFile file)
        {
            m_file = file;
            Debug.Assert(m_file.GetFileExtentCount(DiskImageFile.FileLocationType.UDFFileEntryLocation) == 1);
            BlockAddress = m_file.GetFileExtent(DiskImageFile.FileLocationType.UDFFileEntryLocation, 0).Location;
        }

        protected override ICBType GetIcbType()
        {
            return ICBType.SequenceOfBytes;
        }

        protected override short GetStrategy()
        {
            return 4;
        }

        protected override short GetStrategyParameter()
        {
            return 0;
        }

        protected override short GetMaxEntries()
        {
            return 1;
        }

        protected override long GetInformationLength()
        {
            return m_file.FileSizeInBytes;
        }

        protected override int GetContextExtentCount()
        {
            return m_file.GetFileExtentCount(DiskImageFile.FileLocationType.DataLocation);
        }

        protected override ExtentAd GetContentExtent(int which)
        {
            return m_file.GetFileExtent(DiskImageFile.FileLocationType.DataLocation, which);
        }

        protected override short GetLinkCount()
        {
            return 1;
        }

        protected override DateTime AccessDate(DiskImageCreator creator)
        {
            FileInfo info = new FileInfo(m_file.FileSystemLocation);
            return info.LastAccessTime;
        }

        protected override DateTime ModificationDate(DiskImageCreator creator)
        {
            FileInfo info = new FileInfo(m_file.FileSystemLocation);
            return info.LastWriteTime;
        }

        protected override DateTime AttributeDate(DiskImageCreator creator)
        {
            FileInfo info = new FileInfo(m_file.FileSystemLocation);
            return info.LastAccessTime;
        }
    }
}
