//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics ;

namespace Bwg.Isofs
{
    class FileIdentifierDescriptor : UdfDescriptorTask
    {
        private DiskImageDirectory m_dir;

        [FlagsAttribute]
        public enum FileCharacteristics
        {
            RegularFile = 0x00,
            Existence = 0x01,
            Directory = 0x02,
            Deleted = 0x04,
            Parent = 0x08,
            Metadata = 0x10
        } ;

        public FileIdentifierDescriptor(DiskImageDirectory dir)
        {
            m_dir = dir;
            BlockAddress = dir.GetDirectoryLocation(DiskImageDirectory.DirectoryLocationType.UdfFileIdentifierDescriptor).Location;
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            DiskImageDirectory parent = m_dir.Parent;
            if (parent == null)
                parent = m_dir;

            if (!WriteOneEntry(strm, "", parent))
                return false ;

            foreach (DiskImageElement elem in m_dir)
            {
                if (!WriteOneEntry(strm, elem.GetName(DiskImageElement.NameType.UDF), elem))
                    return false;
            }

            // Now, pad to the end of the current 2k block
            if ((strm.Position % 2048) != 0)
            {
                long pad = 2048 - (strm.Position % 2048);
                strm.WriteZeros((int)pad);
            }

            return true;
        }

        private bool WriteOneEntry(IsoWriteStream strm, string name, DiskImageElement elem)
        {
            byte[] data = new byte[0];
            byte val;
            byte namelen ;


            // Now, populate the data array
            AddShort(ref data, 1);                                                              // File Version Numbre

            val = 0;
            if (elem is DiskImageDirectory && name == "")
            {
                // This is the parent directory entry
                val = (byte)(FileCharacteristics.Directory | FileCharacteristics.Parent);
            }
            else if (elem is DiskImageDirectory)
            {
                // This is a directory
                val = (byte)FileCharacteristics.Directory;
            }
            else
            {
                val = (byte)FileCharacteristics.RegularFile;
            }

            AddByte(ref data, val);                                                             // File Characteristics
            if (name.Length == 0)
                namelen = 0;
            else
            {
                Debug.Assert(name.Length * 2 + 1 <= Byte.MaxValue);
                namelen = (byte)(name.Length * 2 + 1);
            }

            AddByte(ref data, namelen);                                                         // Length of file identifier

            long loc ;
            long size ;
            if (elem is DiskImageDirectory)
            {
                DiskImageDirectory dir = (DiskImageDirectory)elem;
                loc = dir.GetDirectoryLocation(DiskImageDirectory.DirectoryLocationType.UdfFileEntry).Location;
                size = UDFDirectoryAndFileTask.DirectorySizeInBytes(dir);
                size = RoundToNearestBlock(size);
                size = 2048;                                                                    // TODO - fixme
            }
            else
            {
                DiskImageFile file = (DiskImageFile)elem;
                Debug.Assert(file.GetFileExtentCount(DiskImageFile.FileLocationType.UDFFileEntryLocation) == 1);
                loc = file.GetFileExtent(DiskImageFile.FileLocationType.UDFFileEntryLocation, 0).Location;
                size = 2048;
            }

            AddLongExtentAD(ref data, new ExtentAd(loc, size), true);
            AddShort(ref data, 0);

            if (namelen != 0)
                AddDCharacters(ref data, name, namelen);

            while ((data.Length % 4) != 0)
                AddByte(ref data, 0);

            if (!WriteTag(strm, UdfDescriptorTask.FileIdentifierDescriptor, 2, data))
                return false;

            strm.Write(data);

            // Pad to an even multiple of four
            while ((strm.Position % 4) != 0)
                strm.WriteUint8(0);

            return true;
        }
    }
}
