//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.ComponentModel;
using System.Diagnostics;
using Bwg.Logging ;

namespace Bwg.Isofs
{
    class FileTask : CreatorTask
    {
        public const long BlocksPerAllocationSegment = 524287;
        private IList<DiskImageFile> m_file_list;

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            bool ret = true;
            m_file_list = new List<DiskImageFile>();
            creator.RootDirectory.GetFiles(ref m_file_list, true);

            foreach (DiskImageFile f in m_file_list)
            {
                string elemname = Path.GetFileName(f.FileSystemLocation);

                if (!File.Exists(f.FileSystemLocation))
                {
                    string str = "the file '" + f.FileSystemLocation + "' has been removed, the ISO/UDF image cannot be created";
                    UserMessage msg = new UserMessage(UserMessage.Category.Error, 0, str) ;
                    creator.MyLogger.LogMessage(msg);

                    ret = false;
                }

                long count = f.FileSizeInBlocks / BlocksPerAllocationSegment;
                if ((f.FileSizeInBlocks % BlocksPerAllocationSegment) != 0)
                    count++;
                long totalblocks = f.FileSizeInBlocks;
                long totalbytes = f.FileSizeInBytes;

                if (count != 0)
                {
                    for (int j = 0; j < count; j++)
                    {
                        long thissegbytes = totalbytes;
                        long thissegblocks = totalblocks;
                        if (thissegblocks > BlocksPerAllocationSegment)
                        {
                            thissegblocks = BlocksPerAllocationSegment;
                            thissegbytes = thissegblocks * 2048;
                        }

                        ExtentAd ex = new ExtentAd(info.CurrentBlock, thissegbytes);
                        f.SetFileExtent(DiskImageFile.FileLocationType.DataLocation, j, ex);
                        info.CurrentBlock += thissegblocks;

                        totalbytes -= thissegbytes;
                        totalblocks -= thissegblocks;
                    }
                }
                else
                {
                    ExtentAd ex = new ExtentAd(info.CurrentBlock, 0);
                    f.SetFileExtent(DiskImageFile.FileLocationType.DataLocation, 0, ex);
                }
                Debug.Assert(totalblocks == 0);
            }

            return ret;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            byte[] buffer = new byte[2048];
            long blocks = 0;

            strm.StartFileCopy();

            foreach (DiskImageFile f in m_file_list)
            {
                IsoWriteStream.CopyFileStatus stat;

                if (ProgressChanged != null)
                {
                    Debug.Assert(blocks < Int32.MaxValue);
                    ProgressChangedEventArgs args = new ProgressChangedEventArgs((int)blocks, f.GetName(DiskImageElement.NameType.Local));
                    ProgressChanged(this, args);
                }

                //
                // Now, stream the file to the target segments.
                //
                Debug.Assert((strm.Position % 2048) == 0);
                stat = strm.CopyFile(f.FileSystemLocation, f.FileSizeInBytes, f.FileSizeInBlocks * 2048);
                blocks += f.FileSizeInBlocks;

                if (stat != IsoWriteStream.CopyFileStatus.FileJustRight)
                {
                    string str = string.Empty;
                    string fullpath = f.GetPath(DiskImageElement.NameType.Local);

                    switch (stat)
                    {
                        case IsoWriteStream.CopyFileStatus.FileNotFoundSentZeros:
                            str = "the file '" + fullpath + "' has been removed, the contents in the disk image are not valid.";
                            break;

                        case IsoWriteStream.CopyFileStatus.FileTooLongTruncated:
                            str = "the file '" + fullpath + "' has increased in size, the contents in the disk image have been truncated";
                            break;

                        case IsoWriteStream.CopyFileStatus.FileTooShortPaddedZeros:
                            str = "the file '" + fullpath + "' has decreased in size, the contents in the disk image have been padded.";
                            break;
                    }
                    UserMessage msg = new UserMessage(UserMessage.Category.Warning, 1, str);
                    creator.MyLogger.LogMessage(msg);
                }
            }

            strm.EndFileCopy();

            return true;
        }
    }
}
