//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    abstract class InformationControlBlockTask : UdfDescriptorTask
    {
        #region public types
        public enum ICBType : byte
        {
            NoType = 0,
            UnallcatedSpaceEntry = 1,
            PartitionIntegrityEntry = 2,
            IndirectEntry = 3,
            Directory = 4,
            SequenceOfBytes = 5,
            BlockSpecialDevice = 6,
            CharacterSpecialDevice = 7,
            ExtendedAttributes = 8,
            FifoFile = 9,
            Socket = 10,
            TerminalEntry = 11,
            SymbolicLink = 12,
            StreamDirectory = 13
        } ;

        [FlagsAttribute]
        public enum Permissions : uint
        {
            OtherExecute = (1 <<  0),
            OtherWrite =   (1 <<  1),
            OtherRead =    (1 <<  2),
            OtherChAttr =  (1 <<  3),
            OtherDelete =  (1 <<  4),
            GroupExecute = (1 <<  5),
            GroupWrite =   (1 <<  6),
            GroupRead =    (1 <<  7),
            GroupChAttr =  (1 <<  8),
            GroupDelete =  (1 <<  9),
            OwnerExecute = (1 << 10),
            OwnerWrite =   (1 << 11),
            OwnerRead =    (1 << 12),
            OwnerChAttr =  (1 << 13),
            OwnerDelete =  (1 << 14),
        } ;
        #endregion

        private static long m_global_unique_id = 0;
        private long m_unique_id;

        #region constructor
        public InformationControlBlockTask()
        {
            m_unique_id = m_global_unique_id++;
        }
        #endregion

        #region absract functions
        protected abstract short GetStrategy();
        protected abstract short GetStrategyParameter();
        protected abstract ICBType GetIcbType();
        protected abstract short GetMaxEntries();
        protected abstract long GetInformationLength();
        protected abstract int GetContextExtentCount();
        protected abstract ExtentAd GetContentExtent(int n);
        protected abstract short GetLinkCount();
        protected abstract DateTime AccessDate(DiskImageCreator creator);
        protected abstract DateTime ModificationDate(DiskImageCreator creator);
        protected abstract DateTime AttributeDate(DiskImageCreator creator);
        #endregion

        #region public functions
        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            throw new Exception("this function is not implemented");
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            byte[] data = new byte[0];
            long len, blocks ;

            AddIcbTag(ref data);                                            // ICB Tag
            AddInt(ref data, -1);                                           // UID
            AddInt(ref data, -1);                                           // GID

            Permissions p = Permissions.OtherRead | Permissions.OtherExecute |
                            Permissions.GroupRead | Permissions.GroupExecute |
                            Permissions.OwnerRead | Permissions.OwnerExecute;
            AddInt(ref data, (int)p);                                       // Permissions
            AddShort(ref data, GetLinkCount());                             // File Link Count
            AddByte(ref data, 0);                                           // Record Format
            AddByte(ref data, 0);                                           // Record display Attributes
            AddInt(ref data, 0);                                            // Record length

            len = GetInformationLength() ;
            AddLong(ref data, len);                                         // Information Length

            blocks = len / 2048 ;
            if ((len % 2048) != 0)
                blocks++ ;
            AddLong(ref data, blocks);                                      // Logical Blocks Recorded
            AddDateTime(ref data, AccessDate(creator));                     // Access Date and Time
            AddDateTime(ref data, ModificationDate(creator));               // Modification date and time
            AddDateTime(ref data, AttributeDate(creator));                  // Attribute Date And Time
            AddInt(ref data, 1);                                            // Checkpoint
            AddZeros(ref data, 16);                                         // Extended Attribute ICB
            AddRegid(ref data, RegIdType.DeveloperID_2);                    // Implementation identifier
            AddLong(ref data, m_unique_id);                                 // Unique ID
            AddInt(ref data, 0);                                            // Length of extended attributes
            AddInt(ref data, GetContextExtentCount() * 16);                 // Length of allocation descriptors

            // Add the allocation descriptors
            for(int i = 0 ; i < GetContextExtentCount() ; i++)
                AddLongExtentAD(ref data, GetContentExtent(i), true);

            return WriteDescriptor(strm, UdfDescriptorTask.FileEntry, 2, data);
        }
        #endregion

        #region private functions
        private void AddIcbTag(ref byte[] data)
        {
            AddInt(ref data, 0);                                            // Not sure why zero is OK
            // AddInt(ref data, m_element.Depth);                           // The depth in the hierarchy
            AddShort(ref data, GetStrategy());                              // Strategy Type
            AddShort(ref data, GetStrategyParameter());                     // Strategy Parameter

            AddShort(ref data, GetMaxEntries());                           // The total number of dirs + files
            AddByte(ref data, 0);                                          // Reserved
            AddByte(ref data, (byte)GetIcbType());                         // Directory

            AddZeros(ref data, 6);

            //if (m_element.Parent == null)
            //{
                //AddZeros(ref data, 6);
            //}
            //else
            //{
                //addr = m_element.Parent.GetLocation(DiskImageElement.DirLocationUDF);
                //AddLbAddr(ref data, addr, 0, true);                         // Add the parent location relative to this partition
             //} 
            AddShort(ref data, 0x21);                                        // Long allocation descriptors + Archive Bit
        }
        #endregion
    }
}
