//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    class LogicalVolumeIntegrityDescriptorTask : SequencedUDFDescriptor
    {
        public int FileCount;
        public int DirectoryCount;
        public CreatorTask MainPartitionData;

        public LogicalVolumeIntegrityDescriptorTask(int seqno):base(seqno)
        {
            MainPartitionData = null;
            FileCount = -1;
            DirectoryCount = -1;
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            info.CurrentBlock++;
            return true;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            byte[] data = new byte[0];

            Debug.Assert(MainPartitionData != null);

            AddDateTime(ref data, creator.Settings.UdfSettings.CreationDateTime);                   // Recording Date And Time
            AddInt(ref data, 1);                                                        // Closed Integrity Descriptor
            AddExtentAD(ref data, ExtentAd.Empty);                                      // Next Integrity Extent

            AddInt(ref data, 0x1aa);                                                    // TODO: Unique ID (32 bits)
            AddZeros(ref data, 28);                                                     // Reserved (24 + 4)

            AddInt(ref data, 1);                                                        // Number of partitions
            AddInt(ref data, 46);                                                       // Length of Implementation Use (46 bytes)

            AddInt(ref data, 0);
            Debug.Assert(MainPartitionData.ExtentAd.Sectors <= Int32.MaxValue);
            AddInt(ref data, (int)MainPartitionData.ExtentAd.Sectors);


            // Add the implementation Use area per the UDF specification
            AddRegid(ref data, RegIdType.DeveloperID_2);
            AddInt(ref data, creator.FileCount);
            AddInt(ref data, creator.DirectoryCount);
            AddShort(ref data, 0x102);
            AddShort(ref data, 0x102);
            AddShort(ref data, 0x102);

            return WriteDescriptor(strm, UdfDescriptorTask.LogicalVolumeIntegrityDescriptor, 2, data);
        }
    }
}
