//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    class PartitionDescriptorTask : SequencedUDFDescriptor
    {
        public CreatorTask PartitionTask;

        public PartitionDescriptorTask(int seqno) : base(seqno)
        {
            PartitionTask = null;
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            info.CurrentBlock++;
            return true;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            byte[] data = new byte[0];

            Debug.Assert(PartitionTask != null);

            AddInt(ref data, VolumeDescriptorSequenceNumber);                           // Volume Descriptor Sequence Number
            AddShort(ref data, 1);                                                      // Partition Flags (1 = space has been allocated)
            AddShort(ref data, 0);                                                      // Partition Number (always 0)
            AddRegid(ref data, RegIdType.NSR02);                                        // Partition Contents
            AddZeros(ref data, 128);                                                    // Partition Contents Use
            AddInt(ref data, 1);                                                        // Access Type (1 = readonly)

            Debug.Assert(PartitionTask.ExtentAd.Location <= Int32.MaxValue);
            AddInt(ref data, (int)PartitionTask.ExtentAd.Location);                     // Partition Starting Location

            Debug.Assert(PartitionTask.ExtentAd.Sectors <= Int32.MaxValue);
            AddInt(ref data, (int)PartitionTask.ExtentAd.Sectors);                      // Partition Length
            AddRegid(ref data, RegIdType.DeveloperID_2);                                // Implementation Identifier
            AddZeros(ref data, 128);                                                    // Implementation Use
            AddZeros(ref data, 156);                                                    // Reserved

            return WriteDescriptor(strm, UdfDescriptorTask.PartitionalDescriptor, 2, data);
        }
    }
}
