//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    class PathTableTask : CreatorTask
    {
        private DiskImageElement.NameType m_nametype;

        public readonly PathTableType Type;
        public enum PathTableType
        {
            LPathTable,
            MPathTable
        } ;

        private int m_size_in_bytes;
        private int m_size_in_blocks;

        public PathTableTask(DiskImageElement.NameType nametype, PathTableType type)
        {
            Type = type;
            m_nametype = nametype;
        }

        public int PathTableSize
        {
            get
            {
                return m_size_in_bytes;
            }
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            ushort i = 1;

            // Get the directories from the hierarchy
            IList<DiskImageDirectory> Table = creator.GetDirectoryList(m_nametype);

            // Initialize to zero
            m_size_in_bytes = 0;

            // Now add in each directory in the hierarchy
            foreach (DiskImageDirectory dir in Table)
            {
                dir.PathTableEntryNumber = i++;
                m_size_in_bytes += dir.GetPathTableEntrySize(m_nametype);
            }

            // Calculate sizes
            m_size_in_blocks = m_size_in_bytes / 2048;
            if ((m_size_in_bytes % 2048) != 0)
                m_size_in_blocks++;

            info.CurrentBlock += m_size_in_blocks;

            return true;
        }

        private void WriteOneEntry(DiskImageDirectory dir, IsoWriteStream strm)
        {
            byte [] name ;

            if (dir.GetName(m_nametype) == "")
            {
                name = new byte[1];
                name[0] = 0;
            }
            else
            {
                name = GetEncoding(m_nametype).GetBytes(dir.GetName(m_nametype));
            }

            strm.WriteUint8((byte)name.GetLength(0));
            strm.WriteUint8(0);

            long loc = dir.GetDirectoryLocation(NameTypeToDirLocationType(m_nametype)).Location;
            Debug.Assert(loc <= Int32.MaxValue);

            if (Type == PathTableType.LPathTable)
                strm.WriteUint32Small((int)loc);
            else
                strm.WriteUint32Big((int)loc);

            if (dir.Parent != null)
            {
                if (Type == PathTableType.LPathTable)
                    strm.WriteUint16Small(dir.Parent.PathTableEntryNumber);
                else
                    strm.WriteUint16Big(dir.Parent.PathTableEntryNumber);
            }
            else
            {
                if (Type == PathTableType.LPathTable)
                    strm.WriteUint16Small(1);
                else
                    strm.WriteUint16Big(1);
            }

            strm.Write(name);
            if ((name.GetLength(0) % 2) != 0)
                strm.WriteUint8(0) ;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            // Now process each directory in the hierarchy
            IList<DiskImageDirectory> dirs = creator.GetDirectoryList(m_nametype);
            foreach (DiskImageDirectory dir in dirs)
                WriteOneEntry(dir, strm);

            strm.WriteZeros(2048 * m_size_in_blocks - m_size_in_bytes);

            return true;
        }
    }
}
