//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    class PrimaryVolumeDescriptorTask : SequencedUDFDescriptor
    {
        public PrimaryVolumeDescriptorTask(int seqno) : base(seqno)
        {
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            info.CurrentBlock++;
            return true;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            byte[] data = new byte[0];

            //
            // Build the descriptor in the data array.  This is required to calculate the CRC that
            // must be located at the beginning of the data in the descriptor tag.
            //
            AddInt(ref data, VolumeDescriptorSequenceNumber);                           // Volume Descriptor Sequence Number
            AddInt(ref data, 0);                                                        // Primary Volume Descriptor Number
            AddDString(ref data, creator.Settings.UdfSettings.VolumeIdentifier, 32, 16);            // Volume Identifier
            AddShort(ref data, 1);                                                      // Volume Sequence Number
            AddShort(ref data, 1);                                                      // Maximum Volume Sequence Number
            AddShort(ref data, 2);                                                      // Interchange Level
            AddShort(ref data, 2);                                                      // Maximum interchange level
            AddInt(ref data, 1);                                                        // Character set list
            AddInt(ref data, 1);                                                        // Maximum character set list
            AddDString(ref data, GenVsid(), 128, 8);                                    // Volume Set Identifier (see UDF 1.02 spec)
            AddCharset(ref data, UdfDescriptorTask.OSTACompressedUnicode);              // Descriptor Character Set
            AddCharset(ref data, UdfDescriptorTask.OSTACompressedUnicode);              // Explanatory Character Set
            AddExtentAD(ref data, ExtentAd.Empty);                                      // Volume Abstract
            AddExtentAD(ref data, ExtentAd.Empty);                                      // Volume Copyright Notice
            AddZeros(ref data, 32);                                                     // Application Identifier
            AddDateTime(ref data, creator.Settings.UdfSettings.CreationDateTime);       // Recording Date and Time
            AddRegid(ref data, RegIdType.DeveloperID_1);                                // Implementation Identifier
            AddZeros(ref data, 64);                                                     // Implementation Use
            AddInt(ref data, 0);                                                        // Predecessor Volume Descriptor Sequence Location
            AddShort(ref data, 0);                                                      // Flags
            AddZeros(ref data, 22);                                                     // Reserved

            // Now write the descriptor to disk
            return WriteDescriptor(strm, UdfDescriptorTask.PrimaryVolumeDescriptor, 2, data);
        }

        private string GenVsid()
        {
            uint timeval = (uint)(DateTime.Now.Ticks & 0xffffffff);
            return timeval.ToString("x8") + "02211219";

        }
    }
}
