//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    class UDFDirectoryAndFileTask : UdfDescriptorTask
    {
        #region private members
        private IList<DiskImageDirectory> m_dirlist;
        private IList<DiskImageFile> m_file_list;
        #endregion

        #region constructor
        public UDFDirectoryAndFileTask()
        {
        }
        #endregion

        #region public methods
        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            // TODO: This is the overall list, sorted per ISO std.  Will this work for UDF?
            m_dirlist = creator.GetDirectoryList(DiskImageElement.NameType.UDF);

            m_file_list = new List<DiskImageFile>();
            creator.RootDirectory.GetFiles(ref m_file_list, true);

            foreach (DiskImageDirectory dir in m_dirlist)
            {
                if (dir.Contains(DiskImageElement.NameType.Local, "OFFICE11"))
                    Debug.WriteLine("The container for OFFICE11 is block " + info.CurrentBlock);
                else if (dir.GetName(DiskImageElement.NameType.Local) == "OFFICE11")
                    Debug.WriteLine("The location of '" + dir.GetPath(DiskImageElement.NameType.Local) + "' is block " + info.CurrentBlock);

                ExtentAd ex;

                ex = new ExtentAd(info.CurrentBlock++, 2048);
                dir.SetDirectoryLocation(DiskImageDirectory.DirectoryLocationType.UdfFileEntry, ex);

                ex = new ExtentAd(info.CurrentBlock, DirectorySizeInBlocks(dir) * 2048);
                dir.SetDirectoryLocation(DiskImageDirectory.DirectoryLocationType.UdfFileIdentifierDescriptor, ex);
                info.CurrentBlock += DirectorySizeInBlocks(dir);
            }

            foreach (DiskImageFile file in m_file_list)
            {
                ExtentAd ex;

                ex = new ExtentAd(info.CurrentBlock, 2048);
                file.SetFileExtent(DiskImageFile.FileLocationType.UDFFileEntryLocation, 0, ex);
                info.CurrentBlock++;
            }

            return true;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            foreach (DiskImageDirectory dir in m_dirlist)
            {
                if (!WriteDirectory(creator, info, strm, dir))
                    return false;
            }

            foreach (DiskImageFile file in m_file_list)
            {
                if (!WriteFile(creator, info, strm, file))
                    return false;
            }

            return true;
        }
        #endregion

        private bool WriteDirectory(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm, DiskImageDirectory dir)
        {
            // Write the descriptor for this directory
            DirectoryEntryDescriptor entry = new DirectoryEntryDescriptor(dir);
            entry.Partition = this.Partition;
            if (!entry.Write(creator, info, strm))
                return false;

            // Now write the FileIdentifierDescriptor
            FileIdentifierDescriptor iddesc = new FileIdentifierDescriptor(dir);
            iddesc.Partition = this.Partition;
            if (!iddesc.Write(creator, info, strm))
                return false;

            return true;
        }

        private bool WriteFile(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm, DiskImageFile file)
        {
            FileEntryDescriptor entry = new FileEntryDescriptor(file);
            entry.Partition = this.Partition;
            if (!entry.Write(creator, info, strm))
                return false;

            return true;
        }

        static public int DirectorySizeInBlocks(DiskImageDirectory dir)
        {
            int totalbytes = DirectorySizeInBytes(dir);

            int blocks = (totalbytes / 2048);
            if ((totalbytes % 2048) != 0)
                blocks++;

            return blocks;
        }

        static public int DirectorySizeInBytes(DiskImageDirectory dir)
        {
            int totalbytes = 40;            // The initial entry for the parent dir (38 rounded to a four byte boundary)

            foreach (DiskImageElement elem in dir)
                totalbytes += DirEntrySizeInBytes(elem);

            return totalbytes;
        }

        static private int DirEntrySizeInBytes(DiskImageElement elem)
        {
            string name = elem.GetName(DiskImageElement.NameType.UDF);
            int size = 38 + name.Length * 2 + 1;
            int blks = size / 4;
            if ((size % 4) != 0)
                blks++;
            return blks * 4;
        }
    }
}
