//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    abstract class UdfDescriptorTask : CreatorTask
    {
        #region descriptor type constants
        public const ushort PrimaryVolumeDescriptor = 1;
        public const ushort AnchorVolumeDescriptorPointer = 2;
        public const ushort VolumeDescriptorPointer = 3;
        public const ushort ImplementationUseVolumeDescriptor = 4;
        public const ushort PartitionalDescriptor = 5;
        public const ushort LogicalVolumeDescriptor = 6;
        public const ushort UnallocatedSpaceDescriptor = 7;
        public const ushort TerminatingDescriptor = 8;
        public const ushort LogicalVolumeIntegrityDescriptor = 9;

        public const ushort FileSetDescriptor = 256;
        public const ushort FileIdentifierDescriptor = 257;
        public const ushort AllocationExtentDescriptor = 258;
        public const ushort IndirectEntry = 259;
        public const ushort TerminalEntry = 260;
        public const ushort FileEntry = 261;
        public const ushort ExtendedAttributeHeaderDescriptor = 262;
        public const ushort UnallocatedSpaceEntry = 263;
        public const ushort SpaceBitmapDescriptor = 264;
        public const ushort PartitionIntegrityEntry = 265;
        public const ushort ExtendedFileEntry = 266;

        public const string OSTACompressedUnicode = "OSTA Compressed Unicode";
        public const string OSTADomain = "OSTA UDF Compliant" ;

        public enum RegIdType
        {
            NSR02,                          // ID for NSR2 type partition
            DeveloperID_1,                  // ID for this program (BwgSoftware BwgBurn) [Suffix 0, 0, 5, 0]
            DeveloperID_2,                  // ID for this program (BwgSoftware BwgBurn) [Suffix 5, 0, 0, 0]
            DomainID_1_02_1,                // ID for the UDF domain V1.02 (OSTA UDF Compliant + 1.02 suffix)
            DomainID_1_02_2,                // ID for the UDF domain V1.02 (OSTA UDF Compliant + 1.02 suffix)
            UDFIdentifier,                  // ID for UDF identification (UDF LV Info)
        } ;
        #endregion

        #region private member variables
        /// <summary>
        /// This task represents the start of the parition to which this task belongs
        /// </summary>
        CreatorTask m_parition_start;

        #endregion

        #region public properties

        /// <summary>
        /// This partition this task belongs to
        /// </summary>
        public CreatorTask Partition
        {
            get
            {
                return m_parition_start;
            }
            set
            {
                m_parition_start = value;
            }
        }

        #endregion

        public UdfDescriptorTask()
        {
            m_parition_start = null;
        }

        #region CRC constants data
        // CRC 010041
        static ushort[] crc_table = new ushort[256]
            {
                0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7,
                0x8108, 0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1AD, 0xE1CE, 0xF1EF,
                0x1231, 0x0210, 0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6,
                0x9339, 0x8318, 0xB37B, 0xA35A, 0xD3BD, 0xC39C, 0xF3FF, 0xE3DE,
                0x2462, 0x3443, 0x0420, 0x1401, 0x64E6, 0x74C7, 0x44A4, 0x5485,
                0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5EE, 0xF5CF, 0xC5AC, 0xD58D,
                0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6, 0x5695, 0x46B4,
                0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7DF, 0xE7FE, 0xD79D, 0xC7BC,
                0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,
                0xC9CC, 0xD9ED, 0xE98E, 0xF9AF, 0x8948, 0x9969, 0xA90A, 0xB92B,
                0x5AF5, 0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12,
                0xDBFD, 0xCBDC, 0xFBBF, 0xEB9E, 0x9B79, 0x8B58, 0xBB3B, 0xAB1A,
                0x6CA6, 0x7C87, 0x4CE4, 0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41,
                0xEDAE, 0xFD8F, 0xCDEC, 0xDDCD, 0xAD2A, 0xBD0B, 0x8D68, 0x9D49,
                0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13, 0x2E32, 0x1E51, 0x0E70,
                0xFF9F, 0xEFBE, 0xDFDD, 0xCFFC, 0xBF1B, 0xAF3A, 0x9F59, 0x8F78,
                0x9188, 0x81A9, 0xB1CA, 0xA1EB, 0xD10C, 0xC12D, 0xF14E, 0xE16F,
                0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
                0x83B9, 0x9398, 0xA3FB, 0xB3DA, 0xC33D, 0xD31C, 0xE37F, 0xF35E,
                0x02B1, 0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256,
                0xB5EA, 0xA5CB, 0x95A8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D,
                0x34E2, 0x24C3, 0x14A0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
                0xA7DB, 0xB7FA, 0x8799, 0x97B8, 0xE75F, 0xF77E, 0xC71D, 0xD73C,
                0x26D3, 0x36F2, 0x0691, 0x16B0, 0x6657, 0x7676, 0x4615, 0x5634,
                0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99C8, 0x89E9, 0xB98A, 0xA9AB,
                0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882, 0x28A3,
                0xCB7D, 0xDB5C, 0xEB3F, 0xFB1E, 0x8BF9, 0x9BD8, 0xABBB, 0xBB9A,
                0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92,
                0xFD2E, 0xED0F, 0xDD6C, 0xCD4D, 0xBDAA, 0xAD8B, 0x9DE8, 0x8DC9,
                0x7C26, 0x6C07, 0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1,
                0xEF1F, 0xFF3E, 0xCF5D, 0xDF7C, 0xAF9B, 0xBFBA, 0x8FD9, 0x9FF8,
                0x6E17, 0x7E36, 0x4E55, 0x5E74, 0x2E93, 0x3EB2, 0x0ED1, 0x1EF0
            };

        #endregion

        protected ushort CalcCrc(byte[] data)
        {
            ushort crc = 0;

            foreach (byte b in data)
                crc = (ushort)(crc_table[((crc >> 8) ^ b) & 0xff] ^ (crc << 8));

            return crc;
        }

        protected byte CalcCheckSum(ushort tag, ushort version, ushort crc, ushort crclen, ushort serial)
        {
            byte sum = 0;
            long blockaddr = BlockAddress;

            if (Partition != null)
                blockaddr -= Partition.BlockAddress;

            Debug.Assert(blockaddr <= Int32.MaxValue);

            sum += (byte)(tag & 0xff);
            sum += (byte)((tag >> 8) & 0xff);
            sum += (byte)(version & 0xff);
            sum += (byte)((version >> 8) & 0xff);
            sum += (byte)(serial & 0xff);
            sum += (byte)((serial >> 8) & 0xff);
            sum += (byte)(crc & 0xff);
            sum += (byte)((crc >> 8) & 0xff);
            sum += (byte)(crclen & 0xff);
            sum += (byte)((crclen >> 8) & 0xff);
            sum += (byte)(blockaddr & 0xff);
            sum += (byte)((blockaddr >> 8) & 0xff);
            sum += (byte)((blockaddr >> 16) & 0xff);
            sum += (byte)((blockaddr >> 24) & 0xff);

            return sum;
        }

        protected bool WriteTag(IsoWriteStream strm, ushort tag, ushort version, byte[] data)
        {
            ushort serial = 1;
            ushort crc = CalcCrc(data);
            byte csum = CalcCheckSum(tag, version, crc, (ushort)data.GetLength(0), serial);
            long offset = 0;

            //
            // If this tag is relative to a partition, get the offset to the start of the
            // parition
            //
            if (m_parition_start != null)
                offset = m_parition_start.BlockAddress;

            Debug.Assert(BlockAddress >= offset);
            Debug.Assert(BlockAddress - offset <= Int32.MaxValue);

            strm.WriteUint16Small(tag);
            strm.WriteUint16Small(version);
            strm.WriteUint8(csum);
            strm.WriteUint8(0);
            strm.WriteUint16Small(serial);
            strm.WriteUint16Small(crc);
            strm.WriteUint16Small((ushort)data.GetLength(0));
            strm.WriteUint32Small((uint)(BlockAddress - offset));

            return true;
        }

        protected bool WriteDescriptor(IsoWriteStream strm, ushort tag, ushort version, byte [] data)
        {
            if (!WriteTag(strm, tag, version, data))
                return false;

            int pad = 2048 - data.GetLength(0) - 16 ;

            Debug.Assert(pad > 0) ;

            strm.Write(data) ;
            strm.WriteZeros(pad) ;

            return true;
        }

        protected void AddByte(ref byte[] data, byte b)
        {
            int len = data.GetLength(0);

            Array.Resize(ref data, len + 1);
            data[len++] = b;
        }

        protected void AddShort(ref byte[] data, short d)
        {
            int len = data.GetLength(0);

            Array.Resize(ref data, len + 2);
            data[len++] = (byte)((d >> 0) & 0xff);
            data[len++] = (byte)((d >> 8) & 0xff);
        }

        protected void AddInt(ref byte[] data, int d)
        {
            int len = data.GetLength(0);

            Array.Resize(ref data, len + 4);
            data[len++] = (byte)((d >> 0) & 0xff);
            data[len++] = (byte)((d >> 8) & 0xff);
            data[len++] = (byte)((d >> 16) & 0xff);
            data[len++] = (byte)((d >> 24) & 0xff);
        }

        protected void AddLong(ref byte[] data, long d)
        {
            int len = data.GetLength(0);

            Array.Resize(ref data, len + 8);
            data[len++] = (byte)((d >> 0) & 0xff);
            data[len++] = (byte)((d >> 8) & 0xff);
            data[len++] = (byte)((d >> 16) & 0xff);
            data[len++] = (byte)((d >> 24) & 0xff);
            data[len++] = (byte)((d >> 32) & 0xff);
            data[len++] = (byte)((d >> 40) & 0xff);
            data[len++] = (byte)((d >> 48) & 0xff);
            data[len++] = (byte)((d >> 56) & 0xff);
        }

        protected void AddDCharacters(ref byte[] data, string str, int strlen)
        {
            Debug.Assert(str != string.Empty);

            int maxstrlen = (strlen - 1) / 2;
            if (str.Length > maxstrlen)
                str = str.Substring(0, maxstrlen);

            int len = data.GetLength(0);
            Array.Resize(ref data, len + strlen);
            data[len++] = 0x10 ;

            byte[] strdata = Encoding.BigEndianUnicode.GetBytes(str);
            Debug.Assert(strdata.GetLength(0) == str.Length * 2);
            strdata.CopyTo(data, len);
        }

        protected void AddDString(ref byte[] data, string str, int strlen, int bits)
        {
            if (str == string.Empty)
            {
                //
                // An empty string is basically a zero block
                //
                AddZeros(ref data, strlen);
            }
            else
            {
                int len = data.GetLength(0);
                int maxstrlen;

                Array.Resize(ref data, len + strlen);

                if (bits == 8)
                {
                    maxstrlen = strlen - 2;
                    if (str.Length > maxstrlen)
                        str = str.Substring(0, maxstrlen);

                    data[len] = 8;
                    data[len + strlen - 1] = (byte)(str.Length + 1);
                    len++;

                    byte[] strdata = Encoding.ASCII.GetBytes(str);
                    Debug.Assert(strdata.GetLength(0) == str.Length);
                    strdata.CopyTo(data, len);
                }
                else
                {
                    maxstrlen = (strlen - 2) / 2;
                    if (str.Length > maxstrlen)
                        str = str.Substring(0, maxstrlen);

                    data[len] = 0x10;
                    data[len + strlen - 1] = (byte)(str.Length * 2 + 1);
                    len++;

                    byte[] strdata = Encoding.BigEndianUnicode.GetBytes(str);
                    Debug.Assert(strdata.GetLength(0) == str.Length * 2);
                    strdata.CopyTo(data, len);
                }
            }
        }

        protected void AddCharset(ref byte[] data, string str)
        {
            Debug.Assert(str.Length < 64);
            int len = data.GetLength(0);

            Array.Resize(ref data, len + 64);
            data[len++] = 0;
            byte[] strdata = Encoding.ASCII.GetBytes(str);
            strdata.CopyTo(data, len);
        }

        protected void AddExtentAD(ref byte[] data, ExtentAd extent)
        {
            Debug.Assert(extent.Bytes < Int32.MaxValue);
            Debug.Assert(extent.Location < Int32.MaxValue);

            AddInt(ref data, (int)extent.Bytes);
            AddInt(ref data, (int)extent.Location);
        }

        protected void AddLongExtentAD(ref byte[] data, ExtentAd extent, bool relative)
        {
            if (extent == ExtentAd.Empty)
            {
                AddZeros(ref data, 16);
            }
            else
            {
                long offset = 0;

                if (Partition != null && relative)
                    offset = Partition.BlockAddress;

                Debug.Assert(extent.Bytes < Int32.MaxValue);
                Debug.Assert(extent.Location < Int32.MaxValue);
                Debug.Assert(extent.Location >= offset);

                AddInt(ref data, (int)extent.Bytes);
                AddInt(ref data, (int)(extent.Location - offset));
                AddZeros(ref data, 8);
            }
        }

        protected void AddRegid(ref byte[] data, RegIdType id)
        {
            int len = data.GetLength(0);
            byte[] suffix = new byte[8] ;
            string idstr = string.Empty ;

            Array.Resize(ref data, len + 32);

            data[len++] = 0;

            switch (id)
            {
                case RegIdType.NSR02:
                    idstr = "+NSR02";
                    break;

                case RegIdType.DeveloperID_1:
                    idstr = "*AHEAD Nero";
                    //idstr = "BwgSoftware BwgBurn";
                    suffix[0] = 0;
                    suffix[1] = 0;
                    suffix[2] = 5;
                    break;

                case RegIdType.DeveloperID_2:
                    idstr = "*AHEAD Nero";
                    //idstr = "BwgSoftware BwgBurn";
                    suffix[0] = 5;
                    suffix[1] = 0;
                    suffix[2] = 0;
                    break;

                case RegIdType.DomainID_1_02_1:
                    idstr = "*OSTA UDF Compliant";
                    suffix[0] = 2;
                    suffix[1] = 1;
                    suffix[2] = 3;
                    break;

                case RegIdType.DomainID_1_02_2:
                    idstr = "*OSTA UDF Compliant";
                    suffix[0] = 2;
                    suffix[1] = 1;
                    suffix[2] = 0;
                    break;

                case RegIdType.UDFIdentifier:
                    idstr = "*UDF LV Info";
                    suffix[0] = 2;
                    suffix[1] = 1;
                    suffix[2] = 5;                      // Windows class OS
                    suffix[3] = 0;                      // Windows OS
                    break;
            }

            byte[] strdata = Encoding.ASCII.GetBytes(idstr);
            strdata.CopyTo(data, len);
            len += 23;

            suffix.CopyTo(data, len);
        }

        protected void AddDateTime(ref byte[] data, DateTime dt)
        {
            ushort v;
            int len = data.GetLength(0);

            Array.Resize(ref data, len + 12);

            DateTime utc = dt.ToUniversalTime();
            TimeSpan sp = dt - utc;
            int secdiff = (int)(sp.TotalMinutes);

            v = (ushort)((secdiff & 0xfff) | 0x1000);
            data[len++] = (byte)(v & 0xff);
            data[len++] = (byte)((v >> 8) & 0xff);

            v = (ushort)dt.Year;
            data[len++] = (byte)(v & 0xff);
            data[len++] = (byte)((v >> 8) & 0xff);

            data[len++] = (byte)dt.Month;
            data[len++] = (byte)dt.Day;
            data[len++] = (byte)dt.Hour;
            data[len++] = (byte)dt.Minute;
            data[len++] = (byte)dt.Second;
            data[len++] = 0 ;
            data[len++] = 0 ;
            data[len++] = 0;
        }

        protected void AddZeros(ref byte[] data, int count)
        {
            int len = data.GetLength(0);
            Array.Resize(ref data, len + count);
        }

        protected void AddLbAddr(ref byte[] data, long block, short part, bool rel)
        {
            Debug.Assert(block >= Partition.BlockAddress);

            if (rel && Partition != null)
                block -= Partition.BlockAddress;

            Debug.Assert(block <= Int32.MaxValue);
            AddInt(ref data, (int)block);
            AddShort(ref data, part);
        }
    }
}
