//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;

namespace Bwg.Isofs
{
    /// <summary>
    /// This class represents those settings associated with the UDF image
    /// </summary>
    [Serializable()]
    public class UdfImageSettings : ICloneable
    {
        #region private member functions
        /// <summary>
        /// If true, this has been changed since last saved
        /// </summary>
        [NonSerialized()]
        private bool m_dirty;

        /// <summary>
        /// The volume identifier, specified in d characters
        /// </summary>
        private string m_volume_identifier;

        /// <summary>
        /// The creation date and time for this volume
        /// </summary>
        private DateTime m_volume_creation_date;

        #endregion

        #region constructor
        /// <summary>
        /// This constructor creates a new image settings object
        /// </summary>
        public UdfImageSettings()
        {
            m_volume_identifier = string.Empty;
            m_volume_creation_date = DateTime.Now;
        }
        #endregion

        #region public properties
        /// <summary>
        /// This property returns true if this class has been changed, but not saved
        /// </summary>
        [BrowsableAttribute(false)]
        public bool Dirty
        {
            get
            {
                return m_dirty;
            }
            set
            {
                m_dirty = value;
            }
        }

        /// <summary>
        /// The volume identifier string
        /// </summary>
        [Description("This string identifies the volume"), Category("Image Properties")]
        public string VolumeIdentifier
        {
            get
            {
                return m_volume_identifier;
            }

            set
            {
                if (!DiskImageContents.CheckValidDChars(value))
                    throw new InvalidIdentifierException(value);

                if (m_volume_identifier != value)
                {
                    m_volume_identifier = value;
                    m_dirty = true;
                }
            }
        }

        /// <summary>
        /// The creation date and time
        /// </summary>
        [Description("This field gives the creation time and date for the volume."), Category("Dates")]
        public DateTime CreationDateTime
        {
            get
            {
                return m_volume_creation_date;
            }
            set
            {
                if (m_volume_creation_date != value)
                {
                    m_volume_creation_date = value;
                    m_dirty = true;
                }
            }
        }
        #endregion

        #region public methods

        /// <summary>
        /// This method create a copy of an image settings object
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            UdfImageSettings settings = new UdfImageSettings();

            settings.CreationDateTime = CreationDateTime;
            settings.VolumeIdentifier = VolumeIdentifier;

            return settings;
        }
        #endregion
    }
}
