//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace Bwg.Utils
{

    public unsafe class ShellIconMgr
    {
        #region external functions
        [DllImport("Shell32.dll", CharSet=CharSet.Unicode)]
        private static extern IntPtr SHGetFileInfo(string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, uint cbFileInfo, uint uFlags);
        #endregion

        #region structures for external functions
        [FlagsAttribute]
        private enum GetInfoFlags : uint
        {
            SHGFI_ICON = 0x000000100,               // get icon
            SHGFI_DISPLAYNAME = 0x000000200,        // get display name
            SHGFI_TYPENAME = 0x000000400,           // get type name
            SHGFI_ATTRIBUTES = 0x000000800,         // get attributes
            SHGFI_ICONLOCATION = 0x000001000,       // get icon location
            SHGFI_EXETYPE = 0x000002000,            // return exe type
            SHGFI_SYSICONINDEX = 0x000004000,       // get system icon index
            SHGFI_LINKOVERLAY = 0x000008000,        // put a link overlay on icon
            SHGFI_SELECTED = 0x000010000,           // show icon in selected state
            SHGFI_ATTR_SPECIFIED = 0x000020000,     // get only specified attributes
            SHGFI_LARGEICON = 0x000000000,          // get large icon
            SHGFI_SMALLICON = 0x000000001,          // get small icon
            SHGFI_OPENICON = 0x000000002,           // get open icon
            SHGFI_SHELLICONSIZE = 0x000000004,      // get shell size icon
            SHGFI_PIDL = 0x000000008,               // pszPath is a pidl
            SHGFI_USEFILEATTRIBUTES = 0x000000010,  // use passed dwFileAttribute
            SHGFI_ADDOVERLAYS = 0x000000020,        // apply the appropriate overlays
            SHGFI_OVERLAYINDEX = 0x000000040        // Get the index of the overlay
        } ;

        [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Unicode)]
        struct SHFILEINFO
        {
            public const int NAMESIZE = 80;
            public IntPtr hIcon;
            public int iIcon;
            public uint dwAttributes;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
            public string szDisplayName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
            public string szTypeName;
        };
        #endregion

        #region private variables
        private ImageList m_image_list ;
        private IDictionary<string, int> m_index_map;
        private int m_open_folder_index;
        private int m_closed_folder_index;
        #endregion

        #region constructor
        public ShellIconMgr()
        {
            m_image_list = new ImageList();

            m_index_map = new Dictionary<string, int>();
            m_image_list.ImageSize = new Size(16, 16);

            m_open_folder_index = -1;
            m_closed_folder_index = -1;
        }
        #endregion

        #region public properties
        public ImageList ImageList
        {
            get
            {
                return m_image_list;
            }
        }
        #endregion

        #region public methods
        public int GetFolderIconIndex(string filename, bool open)
        {
            int index = 0;

            if (filename == string.Empty)
                filename = "C:\\Windows" ;

            if (filename.Length == 3 && Char.IsLetter(filename[0]) && filename[1] == ':' && filename[2] == '\\')
            {
                // This is a drive name, special case
                return AddIcon(GetFolderIcon(filename, false));
            }

            if (open)
            {
                if (m_open_folder_index == -1)
                    m_open_folder_index = AddIcon(GetFolderIcon(filename, true));

                index = m_open_folder_index;
            }
            else
            {
                if (m_closed_folder_index == -1)
                    m_closed_folder_index = AddIcon(GetFolderIcon(filename, false));
            }

            return index;
        }

        public int GetFileIconIndex(string filename)
        {
            int index ;

            string ext = Path.GetExtension(filename);
            if (!m_index_map.TryGetValue(ext, out index))
                index = AddIcon(GetFileIcon(filename)) ;

            return index;
        }
        #endregion

        #region private methods

        private int AddIcon(Icon i)
        {
            m_image_list.Images.Add(i);
            return m_image_list.Images.Count - 1;
        }

        private Icon GetFileIcon(string filename)
        {
            GetInfoFlags flags = GetInfoFlags.SHGFI_USEFILEATTRIBUTES;

            flags |= GetInfoFlags.SHGFI_ICON;
            flags |= GetInfoFlags.SHGFI_SMALLICON;

            SHFILEINFO info = new SHFILEINFO();
            SHGetFileInfo(filename, 0x80, ref info, (uint)Marshal.SizeOf(info), (uint)flags);

            return Icon.FromHandle(info.hIcon);
        }

        private Icon GetFolderIcon(string filename, bool open)
        {
            GetInfoFlags flags = GetInfoFlags.SHGFI_USEFILEATTRIBUTES;

            flags |= GetInfoFlags.SHGFI_ICON;
            flags |= GetInfoFlags.SHGFI_SMALLICON;

            if (open)
                flags |= GetInfoFlags.SHGFI_OPENICON;

            SHFILEINFO info = new SHFILEINFO();
            SHGetFileInfo(filename, 0x10, ref info, (uint)Marshal.SizeOf(info), (uint)flags);

            return Icon.FromHandle(info.hIcon);
        }
        #endregion
    }
}
